@@ -2,7 +2,6 @@ from aws_cdk import (
     Environment,
     Stack,
     aws_iam as iam,
-    aws_s3 as s3,
 )
 from constructs import Construct
 from typing import Any, List
@@ -320,7 +319,8 @@ class SageworksCoreStack(Stack):
             resources=[f"arn:aws:sagemaker:{self.region}:{self.account}:training-job/*"],
         )
 
-    def model_training_log_statement(self) -> iam.PolicyStatement:
+    @staticmethod
+    def model_training_log_statement() -> iam.PolicyStatement:
         """Create a policy statement for log interactions when training SageMaker models.
 
         Returns:
