@@ -1,4 +1,3 @@
-import os
 import boto3
 import aws_cdk as cdk
 from pprint import pprint
@@ -12,6 +11,9 @@ aws_region = session.region_name
 print(f"Account: {aws_account}")
 print(f"Region: {aws_region}")
 
+# When you want a different docker image change this line
+dashboard_image = "public.ecr.aws/m6i5k1r2/sageworks_dashboard:v0_8_3_amd64"
+
 # SageWorks Configuration
 try:
     from sageworks.utils.config_manager import ConfigManager
@@ -41,6 +43,7 @@ SageworksDashboardStack(
     "SageworksDashboard",
     env={"account": aws_account, "region": aws_region},
     props=SageworksDashboardStackProps(
+        dashboard_image=dashboard_image,
         sageworks_bucket=sageworks_bucket,
         sageworks_api_key=sageworks_api_key,
         existing_vpc_id=existing_vpc_id,
