@@ -1,4 +1,5 @@
 """SQL based Outliers: Compute outliers for all the columns in a DataSource using SQL"""
+
 import logging
 import pandas as pd
 
@@ -47,10 +48,6 @@ class Outliers:
         # Get the top N outliers for each outlier group
         outlier_df = self.get_top_n_outliers(outlier_df)
 
-        # Drop duplicates
-        all_except_outlier_group = [col for col in outlier_df.columns if col != "outlier_group"]
-        outlier_df = outlier_df.drop_duplicates(subset=all_except_outlier_group, ignore_index=True)
-
         # Make sure the dataframe isn't too big, if it's too big sample it down
         if len(outlier_df) > 300:
             log.important(f"Outliers DataFrame is too large {len(outlier_df)}, sampling down to 300 rows")
@@ -77,6 +74,13 @@ class Outliers:
         column_stats = data_source.column_stats()
         descriptive_stats = data_source.descriptive_stats()
 
+        # If there are no numeric columns, return None
+        if not descriptive_stats:
+            log.warning("No numeric columns found in the current computation view of the DataSource")
+            log.warning("If the data source was created from a DataFrame, ensure that the DataFrame was properly typed")
+            log.warning("Recommendation: Properly type the DataFrame and recreate the SageWorks artifact")
+            return None
+
         # Get the column names and types from the DataSource
         column_details = data_source.column_details(view="computation")
 
@@ -195,20 +199,20 @@ class Outliers:
             pd.DataFrame: A DataFrame containing the top N outliers for each outlier group
         """
 
-        def get_extreme_values(group):
+        def get_extreme_values(group: pd.DataFrame) -> pd.DataFrame:
             """Helper function to get the top N extreme values from a group."""
-
-            # Get the column and extreme type (high or low)
             col, extreme_type = group.name.rsplit("_", 1)
-
-            # Sort values depending on whether they are 'high' or 'low' outliers
-            group = group.sort_values(by=col, ascending=(extreme_type == "low"))
-
-            return group.head(n)
+            if extreme_type == "low":
+                return group.nsmallest(n, col)
+            else:
+                return group.nlargest(n, col)
 
         # Group by 'outlier_group' and apply the helper function to get top N extreme values
-        top_outliers = outlier_df.groupby("outlier_group").apply(get_extreme_values).reset_index(drop=True)
-        return top_outliers
+        top_outliers = outlier_df.groupby("outlier_group", group_keys=False).apply(
+            get_extreme_values, include_groups=True
+        )
+
+        return top_outliers.reset_index(drop=True)
 
 
 if __name__ == "__main__":
