@@ -14,6 +14,7 @@ from sageworks.aws_service_broker.aws_service_broker import AWSServiceBroker, Se
 from sageworks.utils.config_manager import ConfigManager
 from sageworks.utils.datetime_utils import datetime_string
 from sageworks.utils.aws_utils import num_columns_ds, num_columns_fs, aws_url
+from sageworks.api.pipeline_manager import PipelineManager
 
 
 class Meta:
@@ -37,6 +38,9 @@ class Meta:
         self.aws_broker = AWSServiceBroker()
         self.cm = ConfigManager()
 
+        # Pipeline Manager
+        self.pipeline_manager = PipelineManager()
+
     def account(self) -> dict:
         """Print out the AWS Account Info
 
@@ -174,7 +178,7 @@ class Meta:
             refresh (bool, optional): Force a refresh of the metadata. Defaults to False.
 
         Returns:
-            dict: A summary of the Data Sources in AWS
+            dict: Detailed information about all the Data Sources in AWS
         """
         data = self.aws_broker.get_metadata(ServiceCategory.DATA_CATALOG, force_refresh=refresh)
 
@@ -184,6 +188,65 @@ class Meta:
         # Return the data
         return data
 
+    def views(self, database: str = "sageworks") -> pd.DataFrame:
+        """Get a summary of the all the Views, for the given database, in AWS
+
+        Args:
+            database (str, optional): Glue database. Defaults to 'sageworks'.
+
+        Returns:
+            pd.DataFrame: A summary of all the Views, for the given database, in AWS
+        """
+        view_data = self.views_deep(database=database)
+        view_summary = []
+
+        # Pull in various bits of metadata for each data source
+        for name, info in view_data.items():
+            summary = {
+                "Name": name,
+                "Modified": datetime_string(info.get("UpdateTime")),
+                "Num Columns": num_columns_ds(info),
+            }
+            view_summary.append(summary)
+
+        # Return the summary
+        return pd.DataFrame(view_summary)
+
+    def view_details(self, view_name: str, database: str = "sageworks", refresh: bool = False) -> Union[dict, None]:
+        """Get detailed information about a specific View in AWS
+
+        Args:
+            view_name (str): The name of the View
+            database (str, optional): Glue database. Defaults to 'sageworks'.
+            refresh (bool, optional): Force a refresh of the metadata. Defaults to False.
+
+        Returns:
+            dict: Detailed information about the data source (or None if not found)
+        """
+        data = self.views_deep(database=database, refresh=refresh)
+        return data.get(view_name)
+
+    def views_deep(self, database: str = "sageworks", refresh: bool = False) -> dict:
+        """Get a deeper set of data for the Views in Athena/AWS
+
+        Args:
+            database (str, optional): Glue database. Defaults to 'sageworks'.
+            refresh (bool, optional): Force a refresh of the metadata. Defaults to False.
+
+        Returns:
+            dict:  Detailed information about all the Views in AWS
+        """
+        data = self.aws_broker.get_metadata(ServiceCategory.DATA_CATALOG, force_refresh=refresh)
+
+        # Views are in two databases, 'sageworks' and 'sagemaker_featurestore'
+        if "views" not in data:
+            self.log.warning("No views found in the metadata")
+            return {}
+        data = data["views"][database]
+
+        # Return the data
+        return data
+
     def feature_sets(self, refresh: bool = False) -> pd.DataFrame:
         """Get a summary of the Feature Sets in AWS
 
@@ -249,6 +312,11 @@ class Meta:
         model_summary = []
         for model_group_name, model_list in model_data.items():
 
+            # Sanity check
+            if not model_list:
+                self.log.critical(f"No models found for {model_group_name}")
+                continue
+
             # Get Summary information for the 'latest' model in the model_list
             latest_model = model_list[0]
             sageworks_meta = latest_model.get("sageworks_meta", {})
@@ -344,6 +412,20 @@ class Meta:
         """
         return self.aws_broker.get_metadata(ServiceCategory.ENDPOINTS, force_refresh=refresh)
 
+    def pipelines(self, refresh: bool = False) -> pd.DataFrame:
+        """Get a summary of the SageWorks Pipelines
+
+        Args:
+            refresh (bool, optional): Force a refresh of the metadata. Defaults to False.
+
+        Returns:
+            pd.DataFrame: A summary of the SageWorks Pipelines
+        """
+        data = self.pipeline_manager.list_pipelines()
+
+        # Return the pipelines summary as a DataFrame
+        return pd.DataFrame(data)
+
     def _remove_sageworks_meta(self, data: dict) -> dict:
         """Internal: Recursively remove any keys with 'sageworks_' in them"""
 
@@ -356,6 +438,10 @@ class Meta:
                 summary_data[key] = value
         return summary_data
 
+    def refresh_all_aws_meta(self) -> None:
+        """Force a refresh of all the metadata"""
+        self.aws_broker.get_all_metadata(force_refresh=True)
+
 
 if __name__ == "__main__":
     """Exercise the SageWorks Meta Class"""
@@ -388,6 +474,25 @@ if __name__ == "__main__":
     print("\n\n*** Data Source Details ***")
     pprint(meta.data_source_details("abalone_data"))
 
+    # Get the Views (Data Sources)
+    print("\n\n*** Views (Data Sources) ***")
+    pprint(meta.views("sageworks"))
+
+    # Get the Views (Feature Sets)
+    print("\n\n*** Views (Feature Sets) ***")
+    fs_views = meta.views("sagemaker_featurestore")
+    pprint(fs_views)
+
+    # Get the View Details (Data Source)
+    print("\n\n*** View Details (Data Source) ***")
+    pprint(meta.view_details("abalone_data_display", "sageworks"))
+
+    # Get the View Details (Feature Set)
+    print("\n\n*** View Details (Feature Set) ***")
+    # Grab the first Feature Set View from the DataFrame
+    feature_set_name = fs_views.iloc[0]["Name"]
+    pprint(meta.view_details(feature_set_name, "sagemaker_featurestore"))
+
     # Get the Feature Sets
     print("\n\n*** Feature Sets ***")
     pprint(meta.feature_sets())
@@ -400,6 +505,10 @@ if __name__ == "__main__":
     print("\n\n*** Endpoints ***")
     pprint(meta.endpoints())
 
+    # Get the Pipelines
+    print("\n\n*** Pipelines ***")
+    pprint(meta.pipelines())
+
     # Now do a deep dive on all the Artifacts
     print("\n\n#")
     print("# Deep Dives ***")
