@@ -1,8 +1,10 @@
 """AWSAccountClamp provides logic/functionality over a set of AWS IAM Services"""
 
 import os
-
+import sys
 import boto3
+import watchtower
+import getpass
 import awswrangler as wr
 from botocore.exceptions import (
     ClientError,
@@ -13,17 +15,28 @@ from botocore.client import BaseClient
 from botocore.credentials import RefreshableCredentials
 from botocore.session import get_session
 from sagemaker.session import Session as SageSession
-from datetime import timedelta
 import logging
 
 # SageWorks Imports
 from sageworks.utils.config_manager import ConfigManager, FatalConfigError
+from sageworks.utils.docker_utils import running_on_docker, running_on_ecs
 
 
 class AWSAccountClamp:
+
+    # Initialize Class Attributes
+    log = None
+    cm = None
+    role_name = None
+    sageworks_bucket_name = None
+    account_id = None
+    region = None
+    boto3_session = None
+    instance = None
+
     def __new__(cls):
         """AWSAccountClamp Singleton Pattern"""
-        if not hasattr(cls, "instance"):
+        if cls.instance is None:
             # Initialize class attributes here
             cls.log = logging.getLogger("sageworks")
             cls.log.info("Creating the AWSAccountClamp Singleton...")
@@ -54,9 +67,21 @@ class AWSAccountClamp:
             cls.cm.load_and_check_license(cls.account_id)
             cls.cm.print_license_info()
 
-            # Initialize the boto3 session (this is a refreshable session)
-            cls.session_time_delta = timedelta(minutes=50)
-            cls.boto3_session = cls._init_boto3_session()
+            # Check if this code is running in AWS Lambda, Glue, or already has the SageWorks Role
+            cls.log.info("Checking Execution Environment...")
+            if cls.running_on_lambda() or cls.running_on_glue() or cls.is_sageworks_role():
+                cls.boto3_session = boto3.Session()
+            else:
+                # Assume the SageWorks Role and set up our AWS Session credentials with automatic refresh
+                cls.boto3_session = cls._sageworks_role_boto3_session()
+
+            # Add a Cloud Watch handler to the sageworks logger
+            cls.log.info("Adding CloudWatch Logs Handler...")
+            cls.log_stream_name = cls.determine_log_stream()
+            cls.log.info(f"Log Stream Name: {cls.log_stream_name}")
+            cls.add_cloudwatch_logs_handler()
+
+            # Create the Singleton Instance
             cls.instance = super(AWSAccountClamp, cls).__new__(cls)
 
         # Return the singleton
@@ -119,6 +144,8 @@ class AWSAccountClamp:
         try:
             if cls.role_name in sts.get_caller_identity()["Arn"]:
                 return True
+            else:
+                return False
         except (ClientError, UnauthorizedSSOTokenError, TokenRetrievalError):
             msg = "SageWorks Role Check Failure: Check AWS_PROFILE and/or Renew SSO Token..."
             cls.log.critical(msg)
@@ -127,40 +154,106 @@ class AWSAccountClamp:
     @classmethod
     def sageworks_execution_role_arn(cls):
         """Get the SageWorks Execution Role ARN"""
-        iam = boto3.client("iam")
-        try:
-            role_arn = iam.get_role(RoleName=cls.role_name)["Role"]["Arn"]
-            return role_arn
-        except iam.exceptions.NoSuchEntityException:
-            msg = f"Could Not Find Role {cls.role_name}"
-            cls.log.critical(msg)
-            raise RuntimeError(msg)
-        except UnauthorizedSSOTokenError:
-            msg = "SageWorks Role Check Failure: Check AWS_PROFILE and/or Renew SSO Token..."
-            cls.log.critical(msg)
-            raise RuntimeError(msg)
+        return f"arn:aws:iam::{cls.account_id}:role/{cls.role_name}"
 
     @classmethod
-    def glue_check(cls):
+    def running_on_glue(cls):
         """
         Check if the current execution environment is an AWS Glue job.
 
         Returns:
             bool: True if running in AWS Glue environment, False otherwise.
         """
-        try:
-            import awsglue  # noqa: F401
-
+        # Check if GLUE_VERSION or GLUE_PYTHON_VERSION is in the environment
+        if "GLUE_VERSION" in os.environ or "GLUE_PYTHON_VERSION" in os.environ:
+            cls.log.info("Running in AWS Glue Environment...")
             return True
-        except ImportError:
+        else:
             return False
 
     @classmethod
-    def _session_credentials(cls):
-        """Internal: Set up our AWS Session credentials for automatic refresh"""
+    def running_on_lambda(cls):
+        """
+        Check if the current execution environment is an AWS Lambda function.
+
+        Returns:
+            bool: True if running in AWS Lambda environment, False otherwise.
+        """
+        if "AWS_LAMBDA_FUNCTION_NAME" in os.environ:
+            cls.log.info("Running in AWS Lambda Environment...")
+            return True
+        else:
+            return False
+
+    @staticmethod
+    def get_executable_name(argv):
+        # Extract the script name from argv[0], get the base name, and remove the extension
+        try:
+            script_path = argv[0]
+            base_name = os.path.basename(script_path)
+            executable_name = os.path.splitext(base_name)[0]
+            return executable_name
+        except Exception:
+            return None
+
+    @classmethod
+    def determine_log_stream(cls):
+        """Determine the log stream name based on the environment."""
+        executable_name = cls.get_executable_name(sys.argv)
+        if cls.running_on_lambda():
+            job_name = executable_name or os.environ.get("AWS_LAMBDA_FUNCTION_NAME", "unknown")
+            return f"lambda/{job_name}"
+        elif cls.running_on_glue():
+            job_name = executable_name or os.environ.get("GLUE_JOB_NAME", "unknown")
+            return f"glue/{job_name}"
+        elif running_on_ecs():
+            job_name = executable_name or os.environ.get("ECS_TASK_DEFINITION_FAMILY", "unknown")
+            return f"dashboard/{job_name}"
+        elif running_on_docker():
+            job_name = executable_name or os.environ.get("SERVICE_NAME", "unknown")
+            return f"docker/{job_name}"
+        else:
+            # This should work across platforms, including Windows
+            return f"laptop/{getpass.getuser()}"
+
+    @classmethod
+    def add_cloudwatch_logs_handler(cls):
+        """Add a CloudWatch Logs handler to the logger"""
+        try:
+            # Get the boto3 session from the SageWorks Account Clamp
+            session = cls.boto3_session
+            cloudwatch_client = session.client("logs")
+            cloudwatch_handler = watchtower.CloudWatchLogHandler(
+                log_group="SageWorksLogGroup",
+                stream_name=cls.log_stream_name,
+                boto3_client=cloudwatch_client,
+            )
+            cloudwatch_handler.setFormatter(cls.log.handlers[0].formatter)
+            cls.log.addHandler(cloudwatch_handler)
+        except ClientError as e:
+            cls.log.error(f"Failed to set up CloudWatch Logs handler: {e}")
+
+    @classmethod
+    def _sageworks_role_boto3_session(cls):
+
+        # Create a refreshable credentials object
+        refreshable_credentials = RefreshableCredentials.create_from_metadata(
+            metadata=cls._assume_sageworks_role_session_credentials(),
+            refresh_using=cls._assume_sageworks_role_session_credentials,
+            method="sts-assume-role",
+        )
+        session = get_session()
+        session._credentials = refreshable_credentials
+        refreshable_session = boto3.Session(botocore_session=session)
+
+        return refreshable_session
+
+    @classmethod
+    def _assume_sageworks_role_session_credentials(cls):
+        """Internal: Assume SageWorks Role and set up our AWS Session credentials for automatic refresh"""
 
         # Assume the SageWorks Execution Role and then pull the credentials
-        cls.log.debug("Assuming the SageWorks Execution Role and Refreshing Credentials...")
+        cls.log.important("Assuming the SageWorks Execution Role with Refreshing Credentials...")
         sts = boto3.Session().client("sts")
         response = sts.assume_role(
             RoleArn=cls.sageworks_execution_role_arn(),
@@ -175,23 +268,6 @@ class AWSAccountClamp:
         cls.log.debug(f"Credentials Refreshed: Expires at {credentials['expiry_time']}")
         return credentials
 
-    @classmethod
-    def _init_boto3_session(cls):
-        if cls.is_sageworks_role() or cls.glue_check():
-            return boto3.Session()
-
-        refreshable_credentials = RefreshableCredentials.create_from_metadata(
-            metadata=cls._session_credentials(),
-            refresh_using=cls._session_credentials,
-            method="sts-assume-role",
-        )
-
-        session = get_session()
-        session._credentials = refreshable_credentials
-        refreshable_session = boto3.Session(botocore_session=session)
-
-        return refreshable_session
-
     @classmethod
     def boto_session(cls):
         """Create a *refreshable* AWS/boto session so that clients don't get TOKEN timeouts"""
