@@ -64,6 +64,12 @@ class AWSServiceBroker:
         cm = ConfigManager()
         cls.sageworks_bucket = cm.get_config("SAGEWORKS_BUCKET")
 
+        # Sanity check our SageWorks Bucket
+        if cls.sageworks_bucket is None:
+            cls.log.critical("SAGEWORKS_BUCKET is not defined")
+            cls.log.critical("Run Initial Setup here: https://supercowpowers.github.io/sageworks/getting_started/")
+            sys.exit(1)
+
         # Construct bucket paths
         cls.incoming_data_bucket = "s3://" + cls.sageworks_bucket + "/incoming-data/"
         cls.data_sources_bucket = "s3://" + cls.sageworks_bucket + "/data-sources/"
@@ -114,8 +120,8 @@ class AWSServiceBroker:
         Args:
             category (ServiceCategory): The Category of metadata to Pull
         """
-        max_attempts = 5
-        sleep_times = [1, 2, 4, 8, 16]
+        sleep_times = [1, 2, 4, 8, 16, 32, 64]
+        max_attempts = len(sleep_times)
         for attempt in range(max_attempts):
             try:
                 cls.fresh_cache.set(category, True)
@@ -136,7 +142,9 @@ class AWSServiceBroker:
                     )
                     time.sleep(sleep_times[attempt])
                 else:
-                    cls.log.warning(f"ClientError response: {error.response}")
+                    if error_code == "ThrottlingException":
+                        cls.log.critical(f"AWS Throttling Exception: Failed after {max_attempts} attempts!")
+                    cls.log.critical(f"AWS Response: {error.response}")
                     break  # Exit the loop
 
     @classmethod
@@ -166,7 +174,7 @@ class AWSServiceBroker:
 
         # Is the AWS data stale?
         if cls.fresh_cache.get(category) is None:
-            cls.log.info(f"Async: Metadata for {category} is stale, launching refresh thread...")
+            cls.log.debug(f"Async: Metadata for {category} is stale, launching refresh thread...")
             cls.fresh_cache.set(category, True)
 
             # Submit the data refresh task to the thread pool
