@@ -131,7 +131,7 @@ class Artifact(ABC):
     def sageworks_meta(self) -> dict:
         """Get the SageWorks specific metadata for this Artifact
         Note: This functionality will work for FeatureSets, Models, and Endpoints
-              but not for DataSources. The DataSource (or child) class overrides this method.
+              but not for DataSources/Graphs. DataSource/Graph classes need to override this method.
         """
         # First, check our cache
         meta_data_key = f"{self.uuid}_sageworks_meta"
@@ -241,7 +241,10 @@ class Artifact(ABC):
         # Add the new metadata to the existing metadata
         self.log.info(f"Upserting SageWorks Metadata for Artifact: {aws_arn}...")
         aws_tags = dict_to_aws_tags(new_meta)
-        self.sm_client.add_tags(ResourceArn=aws_arn, Tags=aws_tags)
+        try:
+            self.sm_client.add_tags(ResourceArn=aws_arn, Tags=aws_tags)
+        except Exception as e:
+            self.log.error(f"Error adding metadata to {aws_arn}: {e}")
 
     def remove_sageworks_meta(self, key_to_remove: str):
         """Remove SageWorks specific metadata from this Artifact
