@@ -1,5 +1,7 @@
 """AthenaSource: SageWorks Data Source accessible through Athena"""
 
+from typing import Union
+from io import StringIO
 import pandas as pd
 import awswrangler as wr
 from datetime import datetime
@@ -58,7 +60,7 @@ class AthenaSource(DataSourceAbstract):
             data_uuid, self.get_database(), refresh=force_refresh
         )
         if self.catalog_table_meta is None:
-            self.log.important(f"Unable to find {self.get_database()}:{self.get_table_name()} in Glue Catalogs...")
+            self.log.error(f"Unable to find {self.get_database()}:{self.get_table_name()} in Glue Catalogs...")
 
         # Call superclass post init
         super().__post_init__()
@@ -75,8 +77,9 @@ class AthenaSource(DataSourceAbstract):
     def exists(self) -> bool:
         """Validation Checks for this Data Source"""
 
-        # We're we able to pull AWS Metadata for this table_name?"""
-        if self.catalog_table_meta is None:
+        # Are we able to pull AWS Metadata for this table_name?"""
+        # Do we have a valid catalog_table_meta?
+        if getattr(self, "catalog_table_meta", None) is None:
             self.log.debug(f"AthenaSource {self.get_table_name()} not found in SageWorks Metadata...")
             return False
         return True
@@ -151,7 +154,11 @@ class AthenaSource(DataSourceAbstract):
                     boto3_session=self.boto_session,
                 )
             else:
-                raise e
+                self.log.critical(f"Failed to upsert metadata: {e}")
+                self.log.critical(f"{self.uuid} is Malformed! Delete this Artifact and recreate it!")
+        except Exception as e:
+            self.log.critical(f"Failed to upsert metadata: {e}")
+            self.log.critical(f"{self.uuid} is Malformed! Delete this Artifact and recreate it!")
 
     def size(self) -> float:
         """Return the size of this data in MegaBytes"""
@@ -181,7 +188,7 @@ class AthenaSource(DataSourceAbstract):
         count_df = self.query(
             f'select count(*) AS sageworks_count from "{self.get_database()}"."{self.get_table_name()}"'
         )
-        return count_df["sageworks_count"][0]
+        return count_df["sageworks_count"][0] if count_df is not None else 0
 
     def num_columns(self) -> int:
         """Return the number of columns for this Data Source"""
@@ -195,7 +202,7 @@ class AthenaSource(DataSourceAbstract):
         """Return the column types of the internal AthenaSource"""
         return [item["Type"] for item in self.catalog_table_meta["StorageDescriptor"]["Columns"]]
 
-    def query(self, query: str) -> pd.DataFrame:
+    def query(self, query: str) -> Union[pd.DataFrame, None]:
         """Query the AthenaSource
 
         Args:
@@ -204,19 +211,29 @@ class AthenaSource(DataSourceAbstract):
         Returns:
             pd.DataFrame: The results of the query
         """
-        df = wr.athena.read_sql_query(
-            sql=query,
-            database=self.get_database(),
-            ctas_approach=False,
-            boto3_session=self.boto_session,
-        )
-        scanned_bytes = df.query_metadata["Statistics"]["DataScannedInBytes"]
-        if scanned_bytes > 0:
-            self.log.info(f"Athena Query successful (scanned bytes: {scanned_bytes})")
-        return df
+        self.log.debug(f"Executing Query: {query}...")
+        try:
+            df = wr.athena.read_sql_query(
+                sql=query,
+                database=self.get_database(),
+                ctas_approach=False,
+                boto3_session=self.boto_session,
+            )
+            scanned_bytes = df.query_metadata["Statistics"]["DataScannedInBytes"]
+            if scanned_bytes > 0:
+                self.log.info(f"Athena Query successful (scanned bytes: {scanned_bytes})")
+            return df
+        except wr.exceptions.QueryFailed as e:
+            self.log.critical(f"Failed to execute query: {e}")
+            return None
 
-    def execute_statement(self, query: str):
-        """Execute a non-returning SQL statement in Athena."""
+    def execute_statement(self, query: str, silence_errors: bool = False):
+        """Execute a non-returning SQL statement in Athena
+
+        Args:
+            query (str): The query to run against the AthenaSource
+            silence_errors (bool): Silence errors (default: False)
+        """
         try:
             # Start the query execution
             query_execution_id = wr.athena.start_query_execution(
@@ -229,8 +246,19 @@ class AthenaSource(DataSourceAbstract):
             # Wait for the query to complete
             wr.athena.wait_query(query_execution_id=query_execution_id, boto3_session=self.boto_session)
             self.log.debug(f"Statement executed successfully: {query_execution_id}")
-        except Exception as e:
-            self.log.error(f"Failed to execute statement: {e}")
+        except wr.exceptions.QueryFailed as e:
+            if "AlreadyExistsException" in str(e):
+                self.log.warning(f"Table already exists. Ignoring: {e}")
+            else:
+                if not silence_errors:
+                    self.log.error(f"Failed to execute statement: {e}")
+                raise
+        except botocore.exceptions.ClientError as e:
+            error_code = e.response["Error"]["Code"]
+            if error_code == "InvalidRequestException":
+                self.log.error(f"Invalid Query: {query}")
+            else:
+                self.log.error(f"Failed to execute statement: {e}")
             raise
 
     def s3_storage_location(self) -> str:
@@ -285,13 +313,12 @@ class AthenaSource(DataSourceAbstract):
         # Return the descriptive stats
         return stat_dict
 
-    def outliers_impl(self, scale: float = 1.5, use_stddev=False, recompute: bool = False) -> pd.DataFrame:
+    def outliers_impl(self, scale: float = 1.5, use_stddev=False) -> pd.DataFrame:
         """Compute outliers for all the numeric columns in a DataSource
 
         Args:
             scale (float): The scale to use for the IQR (default: 1.5)
             use_stddev (bool): Use Standard Deviation instead of IQR (default: False)
-            recompute (bool): Recompute the outliers (default: False)
 
         Returns:
             pd.DataFrame: A DataFrame of outliers from this DataSource
@@ -305,17 +332,29 @@ class AthenaSource(DataSourceAbstract):
         sql_outliers = outliers.Outliers()
         return sql_outliers.compute_outliers(self, scale=scale, use_stddev=use_stddev)
 
-    def smart_sample(self) -> pd.DataFrame:
+    def smart_sample(self, recompute: bool = False) -> pd.DataFrame:
         """Get a smart sample dataframe for this DataSource
 
-        Note:
-            smart = sample data + outliers for the DataSource"""
+        Args:
+            recompute (bool): Recompute the smart sample (default: False)
+
+        Returns:
+            pd.DataFrame: A combined DataFrame of sample data + outliers
+        """
+
+        # Check if we have cached smart_sample data
+        storage_key = f"data_source:{self.uuid}:smart_sample"
+        if not recompute and self.data_storage.get(storage_key):
+            return pd.read_json(StringIO(self.data_storage.get(storage_key)))
+
+        # Compute/recompute the smart sample
+        self.log.important(f"Computing Smart Sample {self.uuid}...")
 
         # Outliers DataFrame
-        outlier_rows = self.outliers()
+        outlier_rows = self.outliers(recompute=recompute)
 
         # Sample DataFrame
-        sample_rows = self.sample()
+        sample_rows = self.sample(recompute=recompute)
         sample_rows["outlier_group"] = "sample"
 
         # Combine the sample rows with the outlier rows
@@ -324,6 +363,16 @@ class AthenaSource(DataSourceAbstract):
         # Drop duplicates
         all_except_outlier_group = [col for col in all_rows.columns if col != "outlier_group"]
         all_rows = all_rows.drop_duplicates(subset=all_except_outlier_group, ignore_index=True)
+
+        # Just grab display view + outlier_group
+        display_columns = self.get_display_columns() + ["outlier_group"]
+        display_columns = [col for col in display_columns if col in all_rows.columns]
+        all_rows = all_rows[display_columns]
+
+        # Cache the smart_sample data
+        self.data_storage.set(storage_key, all_rows.to_json())
+
+        # Return the smart_sample data
         return all_rows
 
     def correlations(self, recompute: bool = False) -> dict[dict]:
@@ -458,10 +507,14 @@ class AthenaSource(DataSourceAbstract):
     def delete(self):
         """Delete the AWS Data Catalog Table and S3 Storage Objects"""
 
-        # Make sure the Feature Group exists
+        # Make sure the AthenaSource exists
         if not self.exists():
             self.log.warning(f"Trying to delete a AthenaSource that doesn't exist: {self.get_table_name()}")
 
+        # Delete the Display View
+        if self._display_view:
+            self._display_view.delete()
+
         # Delete Data Catalog Table
         self.log.info(f"Deleting DataCatalog Table: {self.get_database()}.{self.get_table_name()}...")
         wr.catalog.delete_table_if_exists(self.get_database(), self.get_table_name(), boto3_session=self.boto_session)
@@ -474,7 +527,8 @@ class AthenaSource(DataSourceAbstract):
 
             self.log.info(f"Deleting S3 Storage Objects: {s3_path}...")
             wr.s3.delete_objects(s3_path, boto3_session=self.boto_session)
-        except TypeError:
+        except Exception as e:
+            self.log.error(f"Failed to delete S3 Storage Objects: {e}")
             self.log.warning("Malformed Artifact... good thing it's being deleted...")
 
         # Delete any data in the Cache
@@ -487,7 +541,7 @@ if __name__ == "__main__":
     """Exercise the AthenaSource Class"""
 
     # Retrieve a Data Source
-    my_data = AthenaSource("test_data")
+    my_data = AthenaSource("abalone_data")
 
     # Verify that the Athena Data Source exists
     assert my_data.exists()
@@ -569,7 +623,7 @@ if __name__ == "__main__":
 
     # Set the display columns
     print("\n\nDisplay Columns")
-    print(my_data.set_display_columns(my_data.column_names()[:5]))
+    print(my_data.set_display_columns(my_data.column_names()))
 
     # Get the display columns
     print("\n\nDisplay Columns")
