@@ -23,7 +23,7 @@ class DataSourceAbstract(Artifact):
         # Set up our instance attributes
         self._database = database
         self._table_name = data_uuid
-        self._display_columns = None
+        self._display_view = None
 
     def __post_init__(self):
         # Call superclass post_init
@@ -75,32 +75,18 @@ class DataSourceAbstract(Artifact):
         else:
             raise ValueError(f"Unknown column details view: {view}")
 
+    def get_display_view(self):
+        """Get the Display View for this Data Source"""
+        if self._display_view is None:
+            self._create_display_view()
+        return self._display_view
+
     def get_display_columns(self) -> list[str]:
-        """Get the display columns for this Data Source
+        """Get the columns from our display view
         Returns:
-            list[str]: The display columns for this Data Source
+            list[str]: The columns from our display view
         """
-        # Check if we have the display columns in our metadata
-        if self._display_columns is None:
-            self._display_columns = self.sageworks_meta().get("sageworks_display_columns")
-
-        # If we still don't have display columns, try to set them
-        if self._display_columns is None:
-            # Exclude these automatically generated columns
-            exclude_columns = ["write_time", "api_invocation_time", "is_deleted", "event_time", "id"]
-
-            # We're going to remove any excluded columns from the display columns and limit to 30 total columns
-            self._display_columns = [col for col in self.column_names() if col not in exclude_columns][:30]
-
-            # Add the outlier_group column if it exists and isn't already in the display columns
-            if "outlier_group" in self.column_names():
-                self._display_columns = list(set(self._display_columns) + set(["outlier_group"]))
-
-            # Set the display columns in the metadata
-            self.set_display_columns(self._display_columns, onboard=False)
-
-        # Return the display columns
-        return self._display_columns
+        return self.get_display_view().columns()
 
     def set_display_columns(self, display_columns: list[str], onboard: bool = True):
         """Set the display columns for this Data Source
@@ -110,14 +96,19 @@ class DataSourceAbstract(Artifact):
             onboard (bool): Onboard the Data Source after setting the display columns (default: True)
         """
         self.log.important(f"Setting Display Columns...{display_columns}")
-        self._display_columns = display_columns
-        self.upsert_sageworks_meta({"sageworks_display_columns": self._display_columns})
+        self.get_display_view().create_view(column_list=display_columns)
         if onboard:
             self.onboard()
 
+    def _create_display_view(self):
+        """Internal: Create the Display View for this DataSource"""
+        from sageworks.core.views.display_view import DisplayView
+
+        self._display_view = DisplayView(self)
+
     def num_display_columns(self) -> int:
-        """Return the number of display columns for this Data Source"""
-        return len(self._display_columns) if self._display_columns else 0
+        """Return the number of columns for the display view"""
+        return len(self.get_display_columns())
 
     def get_computation_columns(self) -> list[str]:
         return self.get_display_columns()
@@ -202,16 +193,15 @@ class DataSourceAbstract(Artifact):
 
         # No Cache, so we have to compute the outliers
         self.log.info(f"Computing Outliers {self.uuid}...")
-        df = self.outliers_impl()
+        df = self.outliers_impl(scale=scale)
         self.data_storage.set(storage_key, df.to_json())
         return df
 
     @abstractmethod
-    def outliers_impl(self, scale: float = 1.5, recompute: bool = False) -> pd.DataFrame:
+    def outliers_impl(self, scale: float = 1.5) -> pd.DataFrame:
         """Return a DataFrame of outliers from this DataSource
         Args:
             scale (float): The scale to use for the IQR (default: 1.5)
-            recompute (bool): Recompute the outliers (default: False)
         Returns:
             pd.DataFrame: A DataFrame of outliers from this DataSource
         Notes:
@@ -311,6 +301,11 @@ class DataSourceAbstract(Artifact):
         self.log.important(f"Onboarding {self.uuid}...")
         self.set_status("onboarding")
         self.remove_health_tag("needs_onboard")
+
+        # Make sure our views actually exist
+        self.get_display_view().ensure_exists()
+
+        # Compute the sample, column stats, and outliers
         self.sample(recompute=True)
         self.column_stats(recompute=True)
         self.refresh_meta()  # Refresh the meta since outliers needs descriptive_stats and value_counts
