@@ -8,20 +8,30 @@ import numpy as np
 from io import StringIO
 import awswrangler as wr
 from typing import Union
-import shap
 import joblib
+import logging
 
-# Model Performance Scores
-from sklearn.metrics import (
-    mean_absolute_error,
-    r2_score,
-    root_mean_squared_error,
-    precision_recall_fscore_support,
-    median_absolute_error,
-    roc_auc_score,
-    confusion_matrix,
-)
-from sklearn.preprocessing import LabelBinarizer
+# Check if scikit-learn is installed
+try:
+    # Model Performance Scores
+    from sklearn.metrics import (
+        mean_absolute_error,
+        r2_score,
+        median_absolute_error,
+        roc_auc_score,
+        confusion_matrix,
+        precision_recall_fscore_support,
+        root_mean_squared_error,
+    )
+    from sklearn.preprocessing import LabelBinarizer
+
+except ImportError as e:
+    # Initialize the logger
+    log = logging.getLogger("sageworks")
+    msg = "Please install the scikit-learn package: pip install scikit-learn"
+    log.critical(msg)
+    print(msg)  # Optionally print the message
+    raise SystemExit(msg) from e  # Exit the program with the error message
 
 from sagemaker.serializers import CSVSerializer
 from sagemaker.deserializers import CSVDeserializer
@@ -32,7 +42,7 @@ from sageworks.core.artifacts.artifact import Artifact
 from sageworks.core.artifacts.model_core import ModelCore, ModelType
 from sageworks.aws_service_broker.aws_service_broker import ServiceCategory
 from sageworks.utils.endpoint_metrics import EndpointMetrics
-from sageworks.utils.extract_model_artifact import ExtractModelArtifact
+from sageworks.utils.shapley_values import generate_shap_values
 
 
 class EndpointCore(Artifact):
@@ -92,6 +102,7 @@ class EndpointCore(Artifact):
 
         # This is for endpoint error handling later
         self.endpoint_return_columns = None
+        self.endpoint_retry = 0
 
         # Call SuperClass Post Initialization
         super().__post_init__()
@@ -150,63 +161,9 @@ class EndpointCore(Artifact):
             self.remove_health_tag("no_activity")
         return health_issues
 
-    def predict(self, eval_df: pd.DataFrame) -> pd.DataFrame:
-        """Run inference/prediction on the given Feature DataFrame
-        Args:
-            eval_df (pd.DataFrame): DataFrame to run predictions on (must have superset of features)
-        Returns:
-            pd.DataFrame: Return the DataFrame with additional columns, prediction and any _proba
-        """
+    def is_serverless(self) -> bool:
+        """Check if the current endpoint is serverless.
 
-        # Make sure the eval_df has the features used to train the model
-        features = ModelCore(self.model_name).features()
-        if features and not set(features).issubset(eval_df.columns):
-            raise ValueError(f"DataFrame does not contain required features: {features}")
-
-        # Create our Endpoint Predictor Class
-        predictor = Predictor(
-            self.endpoint_name,
-            sagemaker_session=self.sm_session,
-            serializer=CSVSerializer(),
-            deserializer=CSVDeserializer(),
-        )
-
-        # Now split up the dataframe into 500 row chunks, send those chunks to our
-        # endpoint (with error handling) and stitch all the chunks back together
-        df_list = []
-        for index in range(0, len(eval_df), 500):
-            print("Processing...")
-
-            # Compute partial DataFrames, add them to a list, and concatenate at the end
-            partial_df = self._endpoint_error_handling(predictor, eval_df[index : index + 500])
-            df_list.append(partial_df)
-
-        # Concatenate the dataframes
-        combined_df = pd.concat(df_list, ignore_index=True)
-
-        # Convert data to numeric
-        # Note: Since we're using CSV serializers numeric columns often get changed to generic 'object' types
-
-        # Hard Conversion
-        # Note: We explicitly catch exceptions for columns that cannot be converted to numeric
-        converted_df = combined_df.copy()
-        for column in combined_df.columns:
-            try:
-                converted_df[column] = pd.to_numeric(combined_df[column])
-            except ValueError:
-                # If a ValueError is raised, the column cannot be converted to numeric, so we keep it as is
-                pass
-
-        # Soft Conversion
-        # Convert columns to the best possible dtype that supports the pd.NA missing value.
-        converted_df = converted_df.convert_dtypes()
-
-        # Return the Dataframe
-        return converted_df
-
-    def is_serverless(self):
-        """
-        Check if the current endpoint is serverless.
         Returns:
             bool: True if the endpoint is serverless, False otherwise.
         """
@@ -222,59 +179,6 @@ class EndpointCore(Artifact):
 
         return MonitorCore(self.endpoint_name)
 
-    def _endpoint_error_handling(self, predictor, feature_df):
-        """Internal: Method that handles Errors, Retries, and Binary Search for Error Row(s)"""
-
-        # Convert the DataFrame into a CSV buffer
-        csv_buffer = StringIO()
-        feature_df.to_csv(csv_buffer, index=False)
-
-        # Error Handling if the Endpoint gives back an error
-        try:
-            # Send the CSV Buffer to the predictor
-            results = predictor.predict(csv_buffer.getvalue())
-
-            # Construct a DataFrame from the results
-            results_df = pd.DataFrame.from_records(results[1:], columns=results[0])
-
-            # Capture the return columns
-            self.endpoint_return_columns = results_df.columns.tolist()
-
-            # Return the results dataframe
-            return results_df
-
-        except botocore.exceptions.ClientError as err:
-            if err.response["Error"]["Code"] == "ModelError":  # Model Error
-                # Report the error and raise an exception
-                self.log.critical(f"Endpoint prediction error: {err.response.get('Message')}")
-                raise err
-
-            # Base case: DataFrame with 1 Row
-            if len(feature_df) == 1:
-                # If we don't have ANY known good results we're kinda screwed
-                if not self.endpoint_return_columns:
-                    raise err
-
-                # Construct an Error DataFrame (one row of NaNs in the return columns)
-                results_df = self._error_df(feature_df, self.endpoint_return_columns)
-                return results_df
-
-            # Recurse on binary splits of the dataframe
-            num_rows = len(feature_df)
-            split = int(num_rows / 2)
-            first_half = self._endpoint_error_handling(predictor, feature_df[0:split])
-            second_half = self._endpoint_error_handling(predictor, feature_df[split:num_rows])
-            return pd.concat([first_half, second_half], ignore_index=True)
-
-    def _error_df(self, df, all_columns):
-        """Internal: Method to construct an Error DataFrame (a Pandas DataFrame with one row of NaNs)"""
-        # Create a new dataframe with all NaNs
-        error_df = pd.DataFrame(dict(zip(all_columns, [[np.NaN]] * len(self.endpoint_return_columns))))
-        # Now set the original values for the incoming dataframe
-        for column in df.columns:
-            error_df[column] = df[column].values
-        return error_df
-
     def size(self) -> float:
         """Return the size of this data in MegaBytes"""
         return 0.0
@@ -371,26 +275,26 @@ class EndpointCore(Artifact):
         # Return the details
         return details
 
-    def onboard(self, interactive: bool = True) -> bool:
+    def onboard(self, interactive: bool = False) -> bool:
         """This is a BLOCKING method that will onboard the Endpoint (make it ready)
         Args:
-            interactive (bool, optional): If True, will prompt the user for information. (default: True)
+            interactive (bool, optional): If True, will prompt the user for information. (default: False)
         Returns:
             bool: True if the Endpoint is successfully onboarded, False otherwise
         """
-        self.log.important(f"Onboarding {self.uuid}...")
-        self.set_status("onboarding")
-        self.remove_health_tag("needs_onboard")
 
         # Make sure our input is defined
         if self.get_input() == "unknown":
             if interactive:
                 input_model = input("Input Model?: ")
-                self.upsert_sageworks_meta({"sageworks_input": input_model})
-                self.model_name = input_model
             else:
-                self.log.error("Input Model is not defined!")
+                self.log.critical("Input Model is not defined!")
                 return False
+        else:
+            input_model = self.get_input()
+
+        # Now that we have the details, let's onboard the Endpoint with args
+        return self.onboard_with_args(input_model)
 
         # Run a health check and refresh the meta
         time.sleep(2)  # Give the AWS Metadata a chance to update
@@ -400,6 +304,31 @@ class EndpointCore(Artifact):
         self.set_status("ready")
         return True
 
+    def onboard_with_args(self, input_model: str) -> bool:
+        """Onboard the Endpoint with the given arguments
+
+        Args:
+            input_model (str): The input model for this endpoint
+        Returns:
+            bool: True if the Endpoint is successfully onboarded, False otherwise
+        """
+        # Set the status to onboarding
+        self.set_status("onboarding")
+
+        self.upsert_sageworks_meta({"sageworks_input": input_model})
+        self.model_name = input_model
+
+        # Remove the needs_onboard tag
+        self.remove_health_tag("needs_onboard")
+        self.set_status("ready")
+
+        # Run a health check and refresh the meta
+        time.sleep(2)  # Give the AWS Metadata a chance to update
+        self.health_check()
+        self.refresh_meta()
+        self.details(recompute=True)
+        return True
+
     def model_details(self) -> dict:
         """Return the details about the model used in this Endpoint"""
         if self.model_name == "unknown":
@@ -422,19 +351,20 @@ class EndpointCore(Artifact):
             capture (bool, optional): Capture the inference results and metrics (default=False)
         """
 
-        # This Utility needs to be loaded now to avoid circular imports
+        # This import needs to happen here (instead of top of file) to avoid circular imports
         from sageworks.utils.endpoint_utils import fs_evaluation_data
 
         eval_data_df = fs_evaluation_data(self)
         capture_uuid = "training_holdout" if capture else None
         return self.inference(eval_data_df, capture_uuid)
 
-    def inference(self, eval_df: pd.DataFrame, capture_uuid: str = None) -> pd.DataFrame:
+    def inference(self, eval_df: pd.DataFrame, capture_uuid: str = None, id_column: str = None) -> pd.DataFrame:
         """Run inference and compute performance metrics with optional capture
 
         Args:
             eval_df (pd.DataFrame): DataFrame to run predictions on (must have superset of features)
             capture_uuid (str, optional): UUID of the inference capture (default=None)
+            id_column (str, optional): Name of the ID column (default=None)
 
         Returns:
             pd.DataFrame: DataFrame with the inference results
@@ -444,7 +374,7 @@ class EndpointCore(Artifact):
         """
 
         # Run predictions on the evaluation data
-        prediction_df = self.predict(eval_df)
+        prediction_df = self._predict(eval_df)
 
         # Get the target column
         target_column = ModelCore(self.model_name).target()
@@ -457,7 +387,7 @@ class EndpointCore(Artifact):
 
         # Compute the standard performance metrics for this model
         model_type = self.model_type()
-        if model_type == ModelType.REGRESSOR.value:
+        if model_type in [ModelType.REGRESSOR.value, ModelType.QUANTILE_REGRESSOR.value]:
             prediction_df = self.residuals(target_column, prediction_df)
             metrics = self.regression_metrics(target_column, prediction_df)
         elif model_type == ModelType.CLASSIFIER.value:
@@ -474,11 +404,130 @@ class EndpointCore(Artifact):
         # Capture the inference results and metrics
         if capture_uuid is not None:
             description = capture_uuid.replace("_", " ").title()
-            self._capture_inference_results(capture_uuid, prediction_df, target_column, metrics, description)
+            self._capture_inference_results(capture_uuid, prediction_df, target_column, metrics, description, id_column)
 
         # Return the prediction DataFrame
         return prediction_df
 
+    def _predict(self, eval_df: pd.DataFrame) -> pd.DataFrame:
+        """Internal: Run prediction on the given observations in the given DataFrame
+        Args:
+            eval_df (pd.DataFrame): DataFrame to run predictions on (must have superset of features)
+        Returns:
+            pd.DataFrame: Return the DataFrame with additional columns, prediction and any _proba columns
+        """
+
+        # Make sure the eval_df has the features used to train the model
+        features = ModelCore(self.model_name).features()
+        if features and not set(features).issubset(eval_df.columns):
+            raise ValueError(f"DataFrame does not contain required features: {features}")
+
+        # Create our Endpoint Predictor Class
+        predictor = Predictor(
+            self.endpoint_name,
+            sagemaker_session=self.sm_session,
+            serializer=CSVSerializer(),
+            deserializer=CSVDeserializer(),
+        )
+
+        # Now split up the dataframe into 500 row chunks, send those chunks to our
+        # endpoint (with error handling) and stitch all the chunks back together
+        df_list = []
+        for index in range(0, len(eval_df), 500):
+            print("Processing...")
+
+            # Compute partial DataFrames, add them to a list, and concatenate at the end
+            partial_df = self._endpoint_error_handling(predictor, eval_df[index : index + 500])
+            df_list.append(partial_df)
+
+        # Concatenate the dataframes
+        combined_df = pd.concat(df_list, ignore_index=True)
+
+        # Convert data to numeric
+        # Note: Since we're using CSV serializers numeric columns often get changed to generic 'object' types
+
+        # Hard Conversion
+        # Note: We explicitly catch exceptions for columns that cannot be converted to numeric
+        converted_df = combined_df.copy()
+        for column in combined_df.columns:
+            try:
+                converted_df[column] = pd.to_numeric(combined_df[column])
+            except ValueError:
+                # If a ValueError is raised, the column cannot be converted to numeric, so we keep it as is
+                pass
+
+        # Soft Conversion
+        # Convert columns to the best possible dtype that supports the pd.NA missing value.
+        converted_df = converted_df.convert_dtypes()
+
+        # Return the Dataframe
+        return converted_df
+
+    def _endpoint_error_handling(self, predictor, feature_df):
+        """Internal: Method that handles Errors, Retries, and Binary Search for Error Row(s)"""
+
+        # Convert the DataFrame into a CSV buffer
+        csv_buffer = StringIO()
+        feature_df.to_csv(csv_buffer, index=False)
+
+        # Error Handling if the Endpoint gives back an error
+        try:
+            # Send the CSV Buffer to the predictor
+            results = predictor.predict(csv_buffer.getvalue())
+
+            # Construct a DataFrame from the results
+            results_df = pd.DataFrame.from_records(results[1:], columns=results[0])
+
+            # Capture the return columns
+            self.endpoint_return_columns = results_df.columns.tolist()
+
+            # Return the results dataframe
+            return results_df
+
+        except botocore.exceptions.ClientError as err:
+            if err.response["Error"]["Code"] == "ModelError":  # Model Error
+                # Report the error and raise an exception
+                self.log.critical(f"Endpoint prediction error: {err.response.get('Message')}")
+                raise err
+
+            # Base case: DataFrame with 1 Row
+            if len(feature_df) == 1:
+                # If we don't have ANY known good results we're kinda screwed
+                if not self.endpoint_return_columns:
+                    raise err
+
+                # Construct an Error DataFrame (one row of NaNs in the return columns)
+                results_df = self._error_df(feature_df, self.endpoint_return_columns)
+                return results_df
+
+            # Recurse on binary splits of the dataframe
+            num_rows = len(feature_df)
+            split = int(num_rows / 2)
+            first_half = self._endpoint_error_handling(predictor, feature_df[0:split])
+            second_half = self._endpoint_error_handling(predictor, feature_df[split:num_rows])
+            return pd.concat([first_half, second_half], ignore_index=True)
+
+        # Catch the botocore.errorfactory.ModelNotReadyException
+        # Note: This is a SageMaker specific error that sometimes occurs
+        #       when the endpoint hasn't been used in a long time.
+        except botocore.errorfactory.ModelNotReadyException as err:
+            if self.endpoint_retry >= 3:
+                raise err
+            self.endpoint_retry += 1
+            self.log.critical(f"Endpoint model not ready: {err}")
+            self.log.critical("Waiting and Retrying...")
+            time.sleep(30)
+            return self._endpoint_error_handling(predictor, feature_df)
+
+    def _error_df(self, df, all_columns):
+        """Internal: Method to construct an Error DataFrame (a Pandas DataFrame with one row of NaNs)"""
+        # Create a new dataframe with all NaNs
+        error_df = pd.DataFrame(dict(zip(all_columns, [[np.NaN]] * len(self.endpoint_return_columns))))
+        # Now set the original values for the incoming dataframe
+        for column in df.columns:
+            error_df[column] = df[column].values
+        return error_df
+
     def _capture_inference_results(
         self,
         capture_uuid: str,
@@ -486,6 +535,7 @@ class EndpointCore(Artifact):
         target_column: str,
         metrics: pd.DataFrame,
         description: str,
+        id_column: str = None,
     ):
         """Internal: Capture the inference results and metrics to S3
 
@@ -495,6 +545,7 @@ class EndpointCore(Artifact):
             target_column (str): Name of the target column
             metrics (pd.DataFrame): DataFrame with the performance metrics
             description (str): Description of the inference results
+            id_column (str, optional): Name of the ID column (default=None)
         """
 
         # Compute a dataframe hash (just use the last 8)
@@ -520,11 +571,22 @@ class EndpointCore(Artifact):
         self.log.info(f"Writing metrics to {inference_capture_path}/inference_metrics.csv")
         wr.s3.to_csv(metrics, f"{inference_capture_path}/inference_metrics.csv", index=False)
 
-        # Write the predictions to our S3 Model Inference Folder (just the target and prediction columns)
-        self.log.info(f"Writing predictions to {inference_capture_path}/inference_predictions.csv")
+        # Grab the target column, prediction column, any _proba columns, and the ID column (if present)
         prediction_col = "prediction" if "prediction" in pred_results_df.columns else "predictions"
         output_columns = [target_column, prediction_col]
+
+        # Add any _proba columns to the output columns
         output_columns += [col for col in pred_results_df.columns if col.endswith("_proba")]
+
+        # Add any quantile columns to the output columns
+        output_columns += [col for col in pred_results_df.columns if col.startswith("q_") or col.startswith("qr_")]
+
+        # Add the ID column
+        if id_column and id_column in pred_results_df.columns:
+            output_columns.append(id_column)
+
+        # Write the predictions to our S3 Model Inference Folder
+        self.log.info(f"Writing predictions to {inference_capture_path}/inference_predictions.csv")
         subset_df = pred_results_df[output_columns]
         wr.s3.to_csv(subset_df, f"{inference_capture_path}/inference_predictions.csv", index=False)
 
@@ -536,43 +598,8 @@ class EndpointCore(Artifact):
             # Note: Unlike other dataframes here, we want to write the index (labels) to the CSV
             wr.s3.to_csv(conf_mtx, f"{inference_capture_path}/inference_cm.csv", index=True)
 
-        #
         # Generate SHAP values for our Prediction Dataframe
-        #
-
-        # Grab the model artifact from AWS
-        model_artifact = ExtractModelArtifact(self.endpoint_name).get_model_artifact()
-
-        # Get the exact features used to train the model
-        model_features = model_artifact.feature_names_in_
-        X_pred = pred_results_df[model_features]
-
-        # Compute the SHAP values
-        try:
-            shap_vals = self.shap_values(model_artifact, X_pred)
-
-            # Multiple shap vals CSV for classifiers
-            if model_type == ModelType.CLASSIFIER.value:
-                # Need a separate shapley values CSV for each class
-                for i, class_shap_vals in enumerate(shap_vals):
-                    df_shap = pd.DataFrame(class_shap_vals, columns=X_pred.columns)
-
-                    # Write shap vals to S3 Model Inference Folder
-                    shap_file_path = f"{inference_capture_path}/inference_shap_values_class_{i}.csv"
-                    self.log.info(f"Writing SHAP values to {shap_file_path}")
-                    wr.s3.to_csv(df_shap, shap_file_path, index=False)
-
-            # Single shap vals CSV for regressors
-            if model_type == ModelType.REGRESSOR.value:
-                # Format shap values into single dataframe
-                df_shap = pd.DataFrame(shap_vals, columns=X_pred.columns)
-
-                # Write shap vals to S3 Model Inference Folder
-                self.log.debug(f"Writing SHAP values to {inference_capture_path}/inference_shap_values.csv")
-                wr.s3.to_csv(df_shap, f"{inference_capture_path}/inference_shap_values.csv", index=False)
-
-        except Exception as e:
-            self.log.error(f"Error computing SHAP values: {e}")
+        generate_shap_values(self.endpoint_name, model_type, pred_results_df, inference_capture_path)
 
         # Now recompute the details for our Model
         self.log.important(f"Recomputing Details for {self.model_name} to show latest Inference Results...")
@@ -584,22 +611,6 @@ class EndpointCore(Artifact):
         self.log.important(f"Recomputing Details for {self.uuid} to show latest Inference Results...")
         self.details(recompute=True)
 
-    @staticmethod
-    def shap_values(model, X: pd.DataFrame) -> np.array:
-        """Compute the SHAP values for this Model
-
-        Args:
-            model (Model): Model object
-            X (pd.DataFrame): DataFrame with the prediction results
-
-        Returns:
-            pd.DataFrame: DataFrame with the SHAP values
-        """
-        # Note: For Tree-based models like decision trees, random forests, XGBoost, LightGBM,
-        explainer = shap.TreeExplainer(model)
-        shap_vals = explainer.shap_values(X)
-        return shap_vals
-
     @staticmethod
     def regression_metrics(target_column: str, prediction_df: pd.DataFrame) -> pd.DataFrame:
         """Compute the performance metrics for this Endpoint
@@ -643,7 +654,7 @@ class EndpointCore(Artifact):
             pd.DataFrame: DataFrame with two new columns called 'residuals' and 'residuals_abs'
         """
         # Sanity Check that this is a regression model
-        if self.model_type() != ModelType.REGRESSOR.value:
+        if self.model_type() not in [ModelType.REGRESSOR.value, ModelType.QUANTILE_REGRESSOR.value]:
             self.log.warning("Residuals are only computed for regression models")
             return prediction_df
 
@@ -654,7 +665,7 @@ class EndpointCore(Artifact):
 
         # Add the residuals and the absolute values to the DataFrame
         prediction_df["residuals"] = y_true - y_pred
-        prediction_df["residuals_abs"] = np.abs(y_true - y_pred)
+        prediction_df["residuals_abs"] = np.abs(prediction_df["residuals"])
         return prediction_df
 
     def classification_metrics(self, target_column: str, prediction_df: pd.DataFrame) -> pd.DataFrame:
@@ -727,14 +738,18 @@ class EndpointCore(Artifact):
         prediction_col = "prediction" if "prediction" in prediction_df.columns else "predictions"
         y_pred = prediction_df[prediction_col]
 
-        # Compute the confusion matrix
-        conf_mtx = confusion_matrix(y_true, y_pred)
+        # Special case for low, medium, high classes
+        if (set(y_true) | set(y_pred)) == {"low", "medium", "high"}:
+            labels = ["low", "medium", "high"]
+        else:
+            labels = sorted(list(set(y_true) | set(y_pred)))
 
-        # Get unique labels
-        labels = sorted(list(set(y_true) | set(y_pred)))
+        # Compute the confusion matrix
+        conf_mtx = confusion_matrix(y_true, y_pred, labels=labels)
 
         # Create a DataFrame
         conf_mtx_df = pd.DataFrame(conf_mtx, index=labels, columns=labels)
+        conf_mtx_df.index.name = "labels"
         return conf_mtx_df
 
     def endpoint_config_name(self) -> str:
@@ -846,6 +861,9 @@ if __name__ == "__main__":
     print(my_endpoint.created())
     print(my_endpoint.modified())
 
+    # Test onboarding
+    my_endpoint.onboard()
+
     # Get the tags associated with this Endpoint
     print(f"Tags: {my_endpoint.get_tags()}")
 
