@@ -50,7 +50,7 @@ class FeatureSetCore(Artifact):
 
         # Sanity check and then set up our FeatureSet attributes
         if self.feature_meta is None:
-            self.log.important(f"Could not find feature set {self.uuid} within current visibility scope")
+            self.log.error(f"Could not find feature set {self.uuid} within current visibility scope")
             self.data_source = None
             return
         else:
@@ -68,11 +68,16 @@ class FeatureSetCore(Artifact):
         # Spin up our Feature Store
         self.feature_store = FeatureStore(self.sm_session)
 
+        # Initialize our Views
+        self._display_view = None
+        self._training_view = None
+        self._data_quality_view = None
+
         # Call superclass post_init
         super().__post_init__()
 
         # All done
-        self.log.info(f"FeatureSet Initialized: {self.uuid}")
+        self.log.info(f"FeatureSet Initialized: {self.uuid}...")
 
     def refresh_meta(self):
         """Internal: Refresh our internal AWS Feature Store metadata"""
@@ -147,39 +152,57 @@ class FeatureSetCore(Artifact):
             ds_details[col] = fs_details.get(col, dtype)
         return ds_details
 
-        # Not going to use these for now
-        """
-        internal = {
-            "write_time": "Timestamp",
-            "api_invocation_time": "Timestamp",
-            "is_deleted": "Boolean",
-        }
-        details.update(internal)
-        return details
-        """
+    def get_display_view(self):
+        """Get the Display View for this Data Source"""
+        if self._display_view is None:
+            self._create_view("display")
+        return self._display_view
+
+    def get_training_view(self):
+        """Get the Training View for this Data Source"""
+        if self._training_view is None:
+            self._create_view("training")
+        return self._training_view
+
+    def get_data_quality_view(self):
+        """Get the Data Quality View for this Data Source"""
+        if self._data_quality_view is None:
+            self._create_view("data_quality")
+        return self._data_quality_view
 
     def get_display_columns(self) -> list[str]:
-        """Get the display columns for this FeatureSet
-
+        """Get the columns from our display view
         Returns:
-            list[str]: The display columns for this FeatureSet
-
-        Notes:
-            This just pulls the display columns from the underlying DataSource
+            list[str]: The columns from our display view
         """
-        return self.data_source.get_display_columns()
+        return self.get_display_view().columns()
 
-    def set_display_columns(self, display_columns: list[str]):
-        """Set the display columns for this FeatureSet
+    def set_display_columns(self, display_columns: list[str], onboard: bool = True):
+        """Set the display columns for this Data Source
 
         Args:
-            display_columns (list[str]): The display columns for this FeatureSet
-
-        Notes:
-            This just sets the display columns for the underlying DataSource
+            display_columns (list[str]): The display columns for this Data Source
+            onboard (bool): Onboard the Data Source after setting the display columns (default: True)
         """
-        self.data_source.set_display_columns(display_columns)
-        self.onboard()
+        self.log.important(f"Setting Display Columns...{display_columns}")
+        self.get_display_view().create_view(column_list=display_columns)
+        if onboard:
+            self.onboard()
+
+    def _create_view(self, view_type: str):
+        """Internal: Create the specified View for this FeatureSet
+
+        Args:
+            view_type (str): The type of view to create
+        """
+        from sageworks.core.views import DisplayView, TrainingView, DataQualityView
+
+        if view_type == "display":
+            self._display_view = DisplayView(self)
+        elif view_type == "training":
+            self._training_view = TrainingView(self)
+        elif view_type == "data_quality":
+            self._data_quality_view = DataQualityView(self)
 
     def num_columns(self) -> int:
         """Return the number of columns of the Feature Set"""
@@ -239,25 +262,8 @@ class FeatureSetCore(Artifact):
         date_time = datetime.now(timezone.utc).strftime("%Y-%m-%d_%H:%M:%S")
         s3_output_path = self.feature_sets_s3_path + f"/{self.uuid}/datasets/all_{date_time}"
 
-        # Do we have a training view?
-        training_view = self.get_training_view_table()
-        if training_view:
-            self.log.important(f"Creating S3 Training Data from Training View {training_view}...")
-            table_name = training_view
-        else:
-            self.log.warning(f"No Training View found for {self.uuid}, using FeatureSet directly...")
-            table_name = self.athena_table
-
-        # Make a query that gets all the data from the FeatureSet
-        query = f"SELECT * FROM {table_name}"
-
-        """
-        Note: We're going to circle back to this
-        # Get the snapshot query
-        query = self.snapshot_query(table_name=table_name)
-        """
-
         # Make the query
+        query = f"SELECT * FROM {self.get_training_view().view_table_name}"
         athena_query = FeatureGroup(name=self.uuid, sagemaker_session=self.sm_session).athena_query()
         athena_query.run(query, output_location=s3_output_path)
         athena_query.wait()
@@ -267,6 +273,16 @@ class FeatureSetCore(Artifact):
         full_s3_path = s3_output_path + f"/{query_execution['QueryExecution']['QueryExecutionId']}.csv"
         return full_s3_path
 
+    def get_training_data(self) -> pd.DataFrame:
+        """Get the training data for this FeatureSet
+
+        Returns:
+            pd.DataFrame: The training data for this FeatureSet
+        """
+        from sageworks.core.views.view import View, ViewType
+
+        return View(self, ViewType.TRAINING).pull_dataframe()
+
     def snapshot_query(self, table_name: str = None) -> str:
         """An Athena query to get the latest snapshot of features
 
@@ -310,6 +326,9 @@ class FeatureSetCore(Artifact):
         details = self.summary()
         details["aws_url"] = self.aws_url()
 
+        # Store the AWS URL in the SageWorks Metadata
+        self.upsert_sageworks_meta({"aws_url": details["aws_url"]})
+
         # Now get a summary of the underlying DataSource
         details["storage_summary"] = self.data_source.summary()
 
@@ -350,8 +369,8 @@ class FeatureSetCore(Artifact):
         # Delete our underlying DataSource (Data Catalog Table and S3 Storage Objects)
         self.data_source.delete()
 
-        # Delete the training view
-        self.delete_training_view()
+        # Delete any views
+        self.delete_views()
 
         # Feature Sets can often have a lot of cruft so delete the entire bucket/prefix
         s3_delete_path = self.feature_sets_s3_path + f"/{self.uuid}/"
@@ -381,124 +400,39 @@ class FeatureSetCore(Artifact):
             time.sleep(1)
         self.log.info(f"FeatureSet {feature_group.name} successfully deleted")
 
-    def create_default_training_view(self):
-        """Create a default view in Athena that assigns roughly 80% of the data to training"""
-
-        # Create the view name
-        view_name = f"{self.athena_table}_training"
-        self.log.important(f"Creating default Training View {view_name}...")
-
-        # Do we already have a training column?
-        if "training" in self.column_names():
-            create_view_query = f"CREATE OR REPLACE VIEW {view_name} AS SELECT * FROM {self.athena_table}"
-        else:
-            # No training column, so create one:
-            #    Construct the CREATE VIEW query with a simple modulo operation for the 80/20 split
-            #    using self.record_id as the stable identifier for row numbering
-            create_view_query = f"""
-            CREATE OR REPLACE VIEW {view_name} AS
-            SELECT *, CASE
-                WHEN MOD(ROW_NUMBER() OVER (ORDER BY {self.record_id}), 10) < 8 THEN 1  -- Assign 80% to training
-                ELSE 0  -- Assign roughly 20% to validation
-            END AS training
-            FROM {self.athena_table}
-            """
-
-        # Execute the CREATE VIEW query
-        self.data_source.execute_statement(create_view_query)
-
-    def create_training_view(self, id_column: str, hold_out_ids: list[str]):
-        """Create a view in Athena that marks hold out ids for this FeatureSet
+    def set_training_holdouts(self, id_column: str, holdout_ids: list[str]):
+        """Set the hold out ids for the training view for this FeatureSet
 
         Args:
-            id_column (str): The name of the id column in the output DataFrame.
-            hold_out_ids (list[str]): The list of hold out ids.
+            id_column (str): The name of the id column.
+            holdout_ids (list[str]): The list of holdout ids.
         """
 
-        # Create the view name
-        view_name = f"{self.athena_table}_training"
-        self.log.important(f"Creating Training View {view_name}...")
+        # Create the training view
+        self.get_training_view().create_view(id_column, holdout_ids)
 
-        # Format the list of hold out ids for SQL IN clause
-        if hold_out_ids and all(isinstance(id, str) for id in hold_out_ids):
-            formatted_hold_out_ids = ", ".join(f"'{id}'" for id in hold_out_ids)
-        else:
-            formatted_hold_out_ids = ", ".join(map(str, hold_out_ids))
-
-        # Construct the CREATE VIEW query
-        create_view_query = f"""
-        CREATE OR REPLACE VIEW {view_name} AS
-        SELECT *, CASE
-            WHEN {id_column} IN ({formatted_hold_out_ids}) THEN 0
-            ELSE 1
-        END AS training
-        FROM {self.athena_table}
-        """
-
-        # Execute the CREATE VIEW query
-        self.data_source.execute_statement(create_view_query)
-
-    def set_hold_out_ids(self, id_column: str, hold_out_ids: list[str]):
-        """Set the hold out ids for this FeatureSet
-
-        Args:
-            id_column (str): The name of the id column in the output DataFrame.
-            hold_out_ids (list[str]): The list of hold out ids.
-        """
-        self.create_training_view(id_column, hold_out_ids)
-
-    def get_hold_out_ids(self, id_column: str) -> list[str]:
-        """Get the hold out ids for this FeatureSet
-
-        Args:
-            id_column (str): The name of the id column in the output DataFrame.
-
-        Returns:
-            list[str]: The list of hold out ids.
-        """
-        training_view_table = self.get_training_view_table(create=False)
-        if training_view_table is not None:
-            query = f"SELECT {id_column} FROM {training_view_table} WHERE training = 0"
-            hold_out_ids = self.query(query)[id_column].tolist()
-            return hold_out_ids
-        else:
-            return []
-
-    def get_training_view_table(self, create: bool = True) -> Union[str, None]:
+    def get_training_view_table(self) -> Union[str, None]:
         """Get the name of the training view for this FeatureSet
-        Args:
-            create (bool): Create the training view if it doesn't exist (default=True)
+
         Returns:
             str: The name of the training view for this FeatureSet
         """
-        training_view_name = f"{self.athena_table}_training"
-        glue_client = self.boto_session.client("glue")
-        try:
-            glue_client.get_table(DatabaseName=self.athena_database, Name=training_view_name)
-            return training_view_name
-        except glue_client.exceptions.EntityNotFoundException:
-            if not create:
-                return None
-            self.log.warning(f"Training View for {self.uuid} doesn't exist, creating one...")
-            self.create_default_training_view()
-            time.sleep(1)  # Give AWS a second to catch up
-            return training_view_name
+        # Get the training view table name
+        return self.get_training_view().view_table_name
 
-    def delete_training_view(self):
-        """Delete the training view for this FeatureSet"""
-        try:
-            training_view_table = self.get_training_view_table(create=False)
-            if training_view_table is not None:
-                self.log.info(f"Deleting Training View {training_view_table} for {self.uuid}")
-                glue_client = self.boto_session.client("glue")
-                glue_client.delete_table(DatabaseName=self.athena_database, Name=training_view_table)
-        except botocore.exceptions.ClientError as error:
-            # For ResourceNotFound/ValidationException, this is fine, otherwise raise all other exceptions
-            if error.response["Error"]["Code"] in ["ResourceNotFound", "ValidationException"]:
-                self.log.warning(f"Training View for {self.uuid} doesn't exist, nothing to delete...")
-                pass
-            else:
-                raise error
+    def delete_views(self):
+        """Delete any views associated with this FeatureSet"""
+
+        # Delete the views
+        if self._display_view:
+            self._display_view.delete()
+            self._display_view = None
+        if self._training_view:
+            self._training_view.delete()
+            self._training_view = None
+        if self._data_quality_view:
+            self._data_quality_view.delete()
+            self._data_quality_view = None
 
     def descriptive_stats(self, recompute: bool = False) -> dict:
         """Get the descriptive stats for the numeric columns of the underlying DataSource
@@ -531,12 +465,16 @@ class FeatureSetCore(Artifact):
         """
         return self.data_source.outliers(scale=scale, recompute=recompute)
 
-    def smart_sample(self) -> pd.DataFrame:
+    def smart_sample(self, recompute: bool = False) -> pd.DataFrame:
         """Get a SMART sample dataframe from this FeatureSet
+
+        Args:
+            recompute (bool): Recompute the smart sample (default=False)
+
         Returns:
             pd.DataFrame: A combined DataFrame of sample data + outliers
         """
-        return self.data_source.smart_sample()
+        return self.data_source.smart_sample(recompute=recompute)
 
     def anomalies(self) -> pd.DataFrame:
         """Get a set of anomalous data from the underlying DataSource
@@ -636,6 +574,7 @@ class FeatureSetCore(Artifact):
 
 if __name__ == "__main__":
     """Exercise for FeatureSet Class"""
+    from sageworks.core.artifacts.feature_set_core import FeatureSetCore as LocalFeatureSetCore  # noqa: F811
     from pprint import pprint
 
     # Setup Pandas output options
@@ -644,7 +583,7 @@ if __name__ == "__main__":
     pd.set_option("display.width", 1000)
 
     # Grab a FeatureSet object and pull some information from it
-    my_features = FeatureSetCore("test_features")
+    my_features = LocalFeatureSetCore("test_features")
 
     # Call the various methods
     # What's my AWS ARN and URL
@@ -694,29 +633,17 @@ if __name__ == "__main__":
     outlier_df = my_features.outliers()
     print(outlier_df)
 
-    # Test creating a default training view
-    print("Creating default training view...")
-    my_features.create_default_training_view()
-
-    # Test the hold out set functionality with ints
+    # Set the holdout ids for the training view
     print("Setting hold out ids...")
     table = my_features.get_training_view_table()
     df = my_features.query(f"SELECT id, name FROM {table}")
-    my_hold_out_ids = [id for id in df["id"] if id < 20]
-    my_features.create_training_view("id", my_hold_out_ids)
-
-    # Convenience methods to set and get the hold out ids
-    print("Setting hold out ids...")
-    my_features.set_hold_out_ids("id", my_hold_out_ids)
-    print("Getting hold out ids...")
-    hold_output = my_features.get_hold_out_ids("id")
-    print(hold_output)
-    assert set(hold_output) == set(my_hold_out_ids)
+    my_holdout_ids = [id for id in df["id"] if id < 20]
+    my_features.set_training_holdouts("id", my_holdout_ids)
 
     # Test the hold out set functionality with strings
     print("Setting hold out ids (strings)...")
-    my_hold_out_ids = [name for name in df["name"] if int(name.split(" ")[1]) > 80]
-    my_features.create_training_view("name", my_hold_out_ids)
+    my_holdout_ids = [name for name in df["name"] if int(name.split(" ")[1]) > 80]
+    my_features.set_training_holdouts("name", my_holdout_ids)
 
     # Now delete the AWS artifacts associated with this Feature Set
     # print('Deleting SageWorks Feature Set...')
