@@ -26,11 +26,45 @@ class ModelType(Enum):
 
     CLASSIFIER = "classifier"
     REGRESSOR = "regressor"
-    UNSUPERVISED = "unsupervised"
+    CLUSTERER = "clusterer"
     TRANSFORMER = "transformer"
+    QUANTILE_REGRESSOR = "quantile_regressor"
     UNKNOWN = "unknown"
 
 
+class InferenceImage:
+    """Class for retrieving locked Scikit-Learn inference images"""
+
+    image_uris = {
+        ("us-east-1", "sklearn", "1.2.1"): (
+            "683313688378.dkr.ecr.us-east-1.amazonaws.com/"
+            "sagemaker-scikit-learn@sha256:ed242e33af079f334972acd2a7ddf74d13310d3c9a0ef3a0e9b0429ccc104dcd"
+        ),
+        ("us-east-2", "sklearn", "1.2.1"): (
+            "257758044811.dkr.ecr.us-east-2.amazonaws.com/"
+            "sagemaker-scikit-learn@sha256:ed242e33af079f334972acd2a7ddf74d13310d3c9a0ef3a0e9b0429ccc104dcd"
+        ),
+        ("us-west-1", "sklearn", "1.2.1"): (
+            "746614075791.dkr.ecr.us-west-1.amazonaws.com/"
+            "sagemaker-scikit-learn@sha256:ed242e33af079f334972acd2a7ddf74d13310d3c9a0ef3a0e9b0429ccc104dcd"
+        ),
+        ("us-west-2", "sklearn", "1.2.1"): (
+            "246618743249.dkr.ecr.us-west-2.amazonaws.com/"
+            "sagemaker-scikit-learn@sha256:ed242e33af079f334972acd2a7ddf74d13310d3c9a0ef3a0e9b0429ccc104dcd"
+        ),
+    }
+
+    @classmethod
+    def get_image_uri(cls, region, framework, version):
+        key = (region, framework, version)
+        if key in cls.image_uris:
+            return cls.image_uris[key]
+        else:
+            raise ValueError(
+                f"No matching image found for region: {region}, framework: {framework}, version: {version}"
+            )
+
+
 class ModelCore(Artifact):
     """ModelCore: SageWorks ModelCore Class
 
@@ -125,7 +159,7 @@ class ModelCore(Artifact):
             self.remove_health_tag("model_type_unknown")
 
         # Model Performance Metrics
-        if self.performance_metrics() is None:
+        if self.get_inference_metrics() is None:
             health_issues.append("metrics_needed")
         else:
             self.remove_health_tag("metrics_needed")
@@ -138,7 +172,7 @@ class ModelCore(Artifact):
            sagemaker.model.Model: AWS Sagemaker Model object
         """
         return SagemakerModel(
-            model_data=self.model_package_arn(), sagemaker_session=self.sm_session, image_uri=self.model_image()
+            model_data=self.model_package_arn(), sagemaker_session=self.sm_session, image_uri=self.container_image()
         )
 
     def list_inference_runs(self) -> list[str]:
@@ -156,8 +190,8 @@ class ModelCore(Artifact):
         inference_runs.insert(0, "model_training")
         return inference_runs
 
-    def performance_metrics(self, capture_uuid: str = "latest") -> Union[pd.DataFrame, None]:
-        """Retrieve the performance metrics for this model
+    def get_inference_metrics(self, capture_uuid: str = "latest") -> Union[pd.DataFrame, None]:
+        """Retrieve the inference performance metrics for this model
 
         Args:
             capture_uuid (str, optional): Specific capture_uuid or "training" (default: "latest")
@@ -169,8 +203,8 @@ class ModelCore(Artifact):
         """
         # Try to get the auto_capture 'training_holdout' or the training
         if capture_uuid == "latest":
-            metrics_df = self.performance_metrics("training_holdout")
-            return metrics_df if metrics_df is not None else self.performance_metrics("model_training")
+            metrics_df = self.get_inference_metrics("training_holdout")
+            return metrics_df if metrics_df is not None else self.get_inference_metrics("model_training")
 
         # Grab the metrics captured during model training (could return None)
         if capture_uuid == "model_training":
@@ -213,20 +247,6 @@ class ModelCore(Artifact):
                 self.log.warning(f"Confusion Matrix {capture_uuid} not found for {self.model_name}!")
                 return None
 
-    def predictions(self, capture_uuid: str = "training_holdout") -> Union[pd.DataFrame, None]:
-        """Retrieve the predictions for this model
-
-        Args:
-            capture_uuid (str, optional): Specific capture_uuid or "training" (default: "training_holdout")
-        Returns:
-            pd.DataFrame: DataFrame of the Predictions (might be None)
-        """
-        # Grab the metrics from the SageWorks Metadata (try inference first, then training)
-        inference_preds = self.inference_predictions(capture_uuid)
-        if inference_preds is not None:
-            return inference_preds
-        return self.validation_predictions()
-
     def set_input(self, input: str, force: bool = False):
         """Override: Set the input data for this artifact
 
@@ -265,13 +285,13 @@ class ModelCore(Artifact):
         """AWS ARN (Amazon Resource Name) for the Model Package (within the Group)"""
         return self.latest_model["ModelPackageArn"]
 
-    def model_container_info(self) -> dict:
+    def container_info(self) -> dict:
         """Container Info for the Latest Model Package"""
         return self.latest_model["ModelPackageDetails"]["InferenceSpecification"]["Containers"][0]
 
-    def model_image(self) -> str:
+    def container_image(self) -> str:
         """Container Image for the Latest Model Package"""
-        return self.model_container_info()["Image"]
+        return self.container_info()["Image"]
 
     def aws_url(self):
         """The AWS URL for looking at/querying this data source"""
@@ -300,7 +320,7 @@ class ModelCore(Artifact):
         time.sleep(2)  # Give the AWS Metadata a chance to update
         self.endpoint_inference_path = self.get_endpoint_inference_path()
 
-    def get_endpoints(self) -> list[str]:
+    def endpoints(self) -> list[str]:
         """Get the list of registered endpoints for this Model
 
         Returns:
@@ -323,6 +343,22 @@ class ModelCore(Artifact):
             self.log.warning(f"No registered endpoints found for {self.model_name}!")
             return None
 
+    def set_target(self, target_column: str):
+        """Set the target for this Model
+
+        Args:
+            target_column (str): Target column for this Model
+        """
+        self.upsert_sageworks_meta({"sageworks_model_target": target_column})
+
+    def set_features(self, feature_columns: list[str]):
+        """Set the features for this Model
+
+        Args:
+            feature_columns (list[str]): List of feature columns
+        """
+        self.upsert_sageworks_meta({"sageworks_model_features": feature_columns})
+
     def target(self) -> Union[str, None]:
         """Return the target for this Model (if supervised, else None)
 
@@ -377,6 +413,7 @@ class ModelCore(Artifact):
 
         self.log.info("Recomputing Model Details...")
         details = self.summary()
+        details["pipeline"] = self.get_pipeline()
         details["model_type"] = self.model_type.value
         details["model_package_group_arn"] = self.group_arn()
         details["model_package_arn"] = self.model_package_arn()
@@ -385,28 +422,28 @@ class ModelCore(Artifact):
         details["version"] = aws_meta["ModelPackageVersion"]
         details["status"] = aws_meta["ModelPackageStatus"]
         details["approval_status"] = aws_meta["ModelApprovalStatus"]
-        details["image"] = self.model_image().split("/")[-1]  # Shorten the image uri
+        details["image"] = self.container_image().split("/")[-1]  # Shorten the image uri
 
         # Grab the inference and container info
         package_details = aws_meta["ModelPackageDetails"]
         inference_spec = package_details["InferenceSpecification"]
-        container_info = self.model_container_info()
+        container_info = self.container_info()
         details["framework"] = container_info.get("Framework", "unknown")
         details["framework_version"] = container_info.get("FrameworkVersion", "unknown")
         details["inference_types"] = inference_spec["SupportedRealtimeInferenceInstanceTypes"]
         details["transform_types"] = inference_spec["SupportedTransformInstanceTypes"]
         details["content_types"] = inference_spec["SupportedContentTypes"]
         details["response_types"] = inference_spec["SupportedResponseMIMETypes"]
-        details["model_metrics"] = self.performance_metrics()
+        details["model_metrics"] = self.get_inference_metrics()
         if self.model_type == ModelType.CLASSIFIER:
             details["confusion_matrix"] = self.confusion_matrix()
             details["predictions"] = None
         else:
             details["confusion_matrix"] = None
-            details["predictions"] = self.predictions()
+            details["predictions"] = self.get_inference_predictions()
 
         # Grab the inference metadata
-        details["inference_meta"] = self.inference_metadata()
+        details["inference_meta"] = self.get_inference_metadata()
 
         # Cache the details
         self.data_storage.set(storage_key, details)
@@ -414,6 +451,19 @@ class ModelCore(Artifact):
         # Return the details
         return details
 
+    # Pipeline for this model
+    def get_pipeline(self) -> str:
+        """Get the pipeline for this model"""
+        return self.sageworks_meta().get("sageworks_pipeline")
+
+    def set_pipeline(self, pipeline: str):
+        """Set the pipeline for this model
+
+        Args:
+            pipeline (str): Pipeline that was used to create this model
+        """
+        self.upsert_sageworks_meta({"sageworks_pipeline": pipeline})
+
     def expected_meta(self) -> list[str]:
         """Metadata we expect to see for this Model when it's ready
         Returns:
@@ -428,11 +478,13 @@ class ModelCore(Artifact):
 
     def _determine_model_type(self):
         """Internal: Determine the Model Type"""
-        model_type = input("Model Type? (classifier, regressor, unsupervised, transformer): ")
+        model_type = input("Model Type? (classifier, regressor, quantile_regressor, unsupervised, transformer): ")
         if model_type == "classifier":
             self._set_model_type(ModelType.CLASSIFIER)
         elif model_type == "regressor":
             self._set_model_type(ModelType.REGRESSOR)
+        elif model_type == "quantile_regressor":
+            self._set_model_type(ModelType.QUANTILE_REGRESSOR)
         elif model_type == "unsupervised":
             self._set_model_type(ModelType.UNSUPERVISED)
         elif model_type == "transformer":
@@ -441,53 +493,94 @@ class ModelCore(Artifact):
             self.log.warning(f"Unknown Model Type {model_type}!")
             self._set_model_type(ModelType.UNKNOWN)
 
-    def onboard(self, interactive: bool = True) -> bool:
-        """This is a BLOCKING method that will onboard the Model (make it ready)
+    def onboard(self, ask_everything=False) -> bool:
+        """This is an interactive method that will onboard the Model (make it ready)
 
         Args:
-            interactive (bool, optional): If True, will prompt the user for information. Defaults to True.
+            ask_everything (bool, optional): Ask for all the details. Defaults to False.
+
         Returns:
             bool: True if the Model is successfully onboarded, False otherwise
         """
         # Set the status to onboarding
         self.set_status("onboarding")
 
-        # Interactive Stuff
-        if interactive:
-            # Determine the Model Type
-            while self.is_model_unknown():
-                self._determine_model_type()
+        # Determine the Model Type
+        while self.is_model_unknown():
+            self._determine_model_type()
 
-            # Determine the Target Column (can be None)
-            if self.target() is None:
-                target_column = input("Target Column? (for unsupervised/transformer just type None): ")
-                if target_column in ["None", "none", ""]:
-                    target_column = None
-                self.upsert_sageworks_meta({"sageworks_model_target": target_column})
+        # Is our input data set?
+        if self.get_input() in ["", "unknown"] or ask_everything:
+            input_data = input("Input Data?: ")
+            if input_data not in ["None", "none", "", "unknown"]:
+                self.set_input(input_data)
 
-            # Determine the Feature Columns
-            if self.features() is None:
-                feature_columns = input("Feature Columns? (use commas): ")
-                feature_columns = [e.strip() for e in feature_columns.split(",")]
-                if feature_columns not in [["None"], ["none"], [""]]:
-                    self.upsert_sageworks_meta({"sageworks_model_features": feature_columns})
+        # Determine the Target Column (can be None)
+        target_column = self.target()
+        if target_column is None or ask_everything:
+            target_column = input("Target Column? (for unsupervised/transformer just type None): ")
+            if target_column in ["None", "none", ""]:
+                target_column = None
 
-            # Registered Endpoints?
-            if not self.get_endpoints():
-                endpoints = input("Register Endpoints? (use commas for multiple): ")
-                endpoints = [e.strip() for e in endpoints.split(",")]
-                if endpoints not in [["None"], ["none"], [""]]:
-                    for endpoint in endpoints:
-                        self.log.info(f"Registering Endpoint {endpoint}...")
-                        self.register_endpoint(endpoint)
+        # Determine the Feature Columns
+        feature_columns = self.features()
+        if feature_columns is None or ask_everything:
+            feature_columns = input("Feature Columns? (use commas): ")
+            feature_columns = [e.strip() for e in feature_columns.split(",")]
+            if feature_columns in [["None"], ["none"], [""]]:
+                feature_columns = None
 
-            # Model Owner?
-            if self.get_owner() in [None, "unknown"]:
-                owner = input("Model Owner: ")
-                if owner in ["None", "none", ""]:
-                    self.set_owner("unknown")
-                else:
-                    self.set_owner(owner)
+        # Registered Endpoints?
+        endpoints = self.endpoints()
+        if not endpoints or ask_everything:
+            endpoints = input("Register Endpoints? (use commas for multiple): ")
+            endpoints = [e.strip() for e in endpoints.split(",")]
+            if endpoints in [["None"], ["none"], [""]]:
+                endpoints = None
+
+        # Model Owner?
+        owner = self.get_owner()
+        if owner in [None, "unknown"] or ask_everything:
+            owner = input("Model Owner: ")
+            if owner in ["None", "none", ""]:
+                owner = "unknown"
+
+        # Now that we have all the details, let's onboard the Model with all the args
+        return self.onboard_with_args(self.model_type, target_column, feature_columns, endpoints, owner)
+
+    def onboard_with_args(
+        self,
+        model_type: ModelType,
+        target_column: str = None,
+        feature_list: list = None,
+        endpoints: list = None,
+        owner: str = None,
+    ) -> bool:
+        """Onboard the Model with the given arguments
+
+        Args:
+            model_type (ModelType): Model Type
+            target_column (str): Target Column
+            feature_list (list): List of Feature Columns
+            endpoints (list, optional): List of Endpoints. Defaults to None.
+            owner (str, optional): Model Owner. Defaults to None.
+        Returns:
+            bool: True if the Model is successfully onboarded, False otherwise
+        """
+        # Set the status to onboarding
+        self.set_status("onboarding")
+
+        # Set All the Details
+        self._set_model_type(model_type)
+        if target_column:
+            self.set_target(target_column)
+        if feature_list:
+            self.set_features(feature_list)
+        if endpoints:
+            for endpoint in endpoints:
+                self.register_endpoint(endpoint)
+        if owner:
+            self.set_owner(owner)
 
         # Load the training metrics and inference metrics
         self._load_training_metrics()
@@ -496,13 +589,13 @@ class ModelCore(Artifact):
 
         # Remove the needs_onboard tag
         self.remove_health_tag("needs_onboard")
+        self.set_status("ready")
 
         # Run a health check and refresh the meta
         time.sleep(2)  # Give the AWS Metadata a chance to update
         self.health_check()
         self.refresh_meta()
         self.details(recompute=True)
-        self.set_status("ready")
         return True
 
     def delete(self):
@@ -523,9 +616,12 @@ class ModelCore(Artifact):
         self.sm_client.delete_model_package_group(ModelPackageGroupName=self.model_name)
 
         # Delete any training artifacts
-        s3_delete_path = f"{self.model_training_path}/"
-        self.log.info(f"Deleting Training S3 Objects {s3_delete_path}")
-        wr.s3.delete_objects(s3_delete_path, boto3_session=self.boto_session)
+        try:
+            s3_delete_path = f"{self.model_training_path}/"
+            self.log.info(f"Deleting Training S3 Objects {s3_delete_path}")
+            wr.s3.delete_objects(s3_delete_path, boto3_session=self.boto_session)
+        except Exception:
+            self.log.warning(f"Could not find/delete training artifacts for {self.model_name}!")
 
         # Delete any data in the Cache
         for key in self.data_storage.list_subkeys(f"model:{self.uuid}:"):
@@ -547,9 +643,9 @@ class ModelCore(Artifact):
             Use the model_type attribute instead
         """
         model_type = self.sageworks_meta().get("sageworks_model_type")
-        if model_type and model_type != "unknown":
+        try:
             return ModelType(model_type)
-        else:
+        except ValueError:
             self.log.warning(f"Could not determine model type for {self.model_name}!")
             return ModelType.UNKNOWN
 
@@ -566,7 +662,7 @@ class ModelCore(Artifact):
                 self.log.warning(f"No training job metrics found for {self.training_job_name}")
                 self.upsert_sageworks_meta({"sageworks_training_metrics": None, "sageworks_training_cm": None})
                 return
-            if self.model_type == ModelType.REGRESSOR:
+            if self.model_type in [ModelType.REGRESSOR, ModelType.QUANTILE_REGRESSOR]:
                 if "timestamp" in df.columns:
                     df = df.drop(columns=["timestamp"])
 
@@ -630,7 +726,7 @@ class ModelCore(Artifact):
         cm_storage = None if inference_cm is None else inference_cm.to_dict("records")
         self.upsert_sageworks_meta({"sageworks_inference_cm": cm_storage})
 
-    def inference_metadata(self, capture_uuid: str = "training_holdout") -> Union[pd.DataFrame, None]:
+    def get_inference_metadata(self, capture_uuid: str = "training_holdout") -> Union[pd.DataFrame, None]:
         """Retrieve the inference metadata for this model
 
         Args:
@@ -668,7 +764,7 @@ class ModelCore(Artifact):
             self.log.info(f"Could not find model inference meta at {s3_path}...")
             return None
 
-    def inference_predictions(self, capture_uuid: str = "training_holdout") -> Union[pd.DataFrame, None]:
+    def get_inference_predictions(self, capture_uuid: str = "training_holdout") -> Union[pd.DataFrame, None]:
         """Retrieve the captured prediction results for this model
 
         Args:
@@ -678,10 +774,16 @@ class ModelCore(Artifact):
             pd.DataFrame: DataFrame of the Captured Predictions (might be None)
         """
         self.log.important(f"Grabbing {capture_uuid} predictions for {self.model_name}...")
+
+        # Special case for model_training
+        if capture_uuid == "model_training":
+            return self._get_validation_predictions()
+
+        # Construct the S3 path for the Inference Predictions
         s3_path = f"{self.endpoint_inference_path}/{capture_uuid}/inference_predictions.csv"
         return pull_s3_data(s3_path)
 
-    def validation_predictions(self) -> Union[pd.DataFrame, None]:
+    def _get_validation_predictions(self) -> Union[pd.DataFrame, None]:
         """Internal: Retrieve the captured prediction results for this model
 
         Returns:
@@ -695,7 +797,7 @@ class ModelCore(Artifact):
     def _extract_training_job_name(self) -> Union[str, None]:
         """Internal: Extract the training job name from the ModelDataUrl"""
         try:
-            model_data_url = self.model_container_info()["ModelDataUrl"]
+            model_data_url = self.container_info()["ModelDataUrl"]
             parsed_url = urllib.parse.urlparse(model_data_url)
             training_job_name = parsed_url.path.lstrip("/").split("/")[0]
             return training_job_name
@@ -764,13 +866,14 @@ class ModelCore(Artifact):
             return [pull_s3_data(f) for f in s3_paths if "inference_shap_values" in f]
 
         # One CSV if regressor
-        if self.model_type == ModelType.REGRESSOR:
+        if self.model_type in [ModelType.REGRESSOR, ModelType.QUANTILE_REGRESSOR]:
             s3_path = f"{shapley_s3_path}/inference_shap_values.csv"
             return pull_s3_data(s3_path)
 
 
 if __name__ == "__main__":
     """Exercise the ModelCore Class"""
+    from pprint import pprint
 
     # Grab a ModelCore object and pull some information from it
     my_model = ModelCore("abalone-regression")
@@ -781,17 +884,22 @@ if __name__ == "__main__":
     print(f"Model Check: {my_model.exists()}")
 
     # Make sure the model is 'ready'
-    my_model.onboard(interactive=False)
+    my_model.onboard()
 
     # Get the ARN of the Model Group
     print(f"Model Group ARN: {my_model.group_arn()}")
     print(f"Model Package ARN: {my_model.arn()}")
 
+    # Get the container info
+    print("Container Info:")
+    pprint(my_model.container_info())
+
     # Get the tags associated with this Model
     print(f"Tags: {my_model.get_tags()}")
 
     # Get the SageWorks metadata associated with this Model
-    print(f"SageWorks Meta: {my_model.sageworks_meta()}")
+    print("SageWorks Meta:")
+    pprint(my_model.sageworks_meta())
 
     # Get creation time
     print(f"Created: {my_model.created()}")
@@ -804,14 +912,14 @@ if __name__ == "__main__":
 
     # Get any captured metrics from the training job
     print("Model Metrics:")
-    print(my_model.performance_metrics())
+    print(my_model.get_inference_metrics())
 
     print("Confusion Matrix: (might be None)")
     print(my_model.confusion_matrix())
 
     # Grab our regression predictions from S3
     print("Captured Predictions: (might be None)")
-    print(my_model.predictions())
+    print(my_model.get_inference_predictions())
 
     # Grab our Shapley values from S3
     print("Shapley Values: (might be None)")
