@@ -10,7 +10,7 @@ from datetime import datetime
 # Local Imports
 from sageworks.core.transforms.transform import Transform, TransformInput, TransformOutput
 from sageworks.core.artifacts.feature_set_core import FeatureSetCore
-from sageworks.core.artifacts.model_core import ModelCore, ModelType
+from sageworks.core.artifacts.model_core import ModelCore, ModelType, InferenceImage
 from sageworks.core.artifacts.artifact import Artifact
 
 
@@ -26,12 +26,13 @@ class FeaturesToModel(Transform):
         ```
     """
 
-    def __init__(self, feature_uuid: str, model_uuid: str, model_type: ModelType):
+    def __init__(self, feature_uuid: str, model_uuid: str, model_type: ModelType = ModelType.UNKNOWN, model_class=None):
         """FeaturesToModel Initialization
         Args:
             feature_uuid (str): UUID of the FeatureSet to use as input
             model_uuid (str): UUID of the Model to create as output
-            model_type (ModelType): ModelType.REGRESSOR or ModelType.CLASSIFIER
+            model_type (ModelType): ModelType.REGRESSOR or ModelType.CLASSIFIER, etc.
+            model_class (str): The class of the model (optional)
         """
 
         # Make sure the model_uuid is a valid name
@@ -40,10 +41,21 @@ class FeaturesToModel(Transform):
         # Call superclass init
         super().__init__(feature_uuid, model_uuid)
 
+        # If the model_type is UNKNOWN the model_class must be specified
+        if model_type == ModelType.UNKNOWN:
+            if model_class is None:
+                msg = "ModelType is UNKNOWN, must specify a model_class!"
+                self.log.critical(msg)
+                raise ValueError(msg)
+            else:
+                self.log.info("ModelType is UNKNOWN, using model_class to determine the type...")
+                model_type = self._determine_model_type(model_class)
+
         # Set up all my instance attributes
         self.input_type = TransformInput.FEATURE_SET
         self.output_type = TransformOutput.MODEL
         self.model_type = model_type
+        self.model_class = model_class
         self.estimator = None
         self.model_script_dir = None
         self.model_description = None
@@ -52,40 +64,123 @@ class FeaturesToModel(Transform):
         self.target_column = None
         self.class_labels = None
 
-    def generate_model_script(
-        self, target_column: str, feature_list: list[str], model_type: ModelType, train_all_data: bool
-    ) -> str:
+    def _determine_model_type(self, model_class: str) -> ModelType:
+        """Determine the ModelType from the model_class
+        Args:
+            model_class (str): The class of the model
+        Returns:
+            ModelType: The determined ModelType
+        """
+        model_class_lower = model_class.lower()
+
+        # Direct mapping for specific models
+        specific_model_mapping = {
+            "logisticregression": ModelType.CLASSIFIER,
+            "linearregression": ModelType.REGRESSOR,
+            "ridge": ModelType.REGRESSOR,
+            "lasso": ModelType.REGRESSOR,
+            "elasticnet": ModelType.REGRESSOR,
+            "bayesianridge": ModelType.REGRESSOR,
+            "svc": ModelType.CLASSIFIER,
+            "svr": ModelType.REGRESSOR,
+            "gaussiannb": ModelType.CLASSIFIER,
+            "kmeans": ModelType.CLUSTERER,
+            "dbscan": ModelType.CLUSTERER,
+            "meanshift": ModelType.CLUSTERER,
+        }
+
+        if model_class_lower in specific_model_mapping:
+            return specific_model_mapping[model_class_lower]
+
+        # General pattern matching
+        if "regressor" in model_class_lower:
+            return ModelType.REGRESSOR
+        elif "classifier" in model_class_lower:
+            return ModelType.CLASSIFIER
+        elif "quantile" in model_class_lower:
+            return ModelType.QUANTILE_REGRESSOR
+        elif "cluster" in model_class_lower:
+            return ModelType.CLUSTERER
+        elif "transform" in model_class_lower:
+            return ModelType.TRANSFORMER
+        else:
+            self.log.critical(f"Unknown ModelType for model_class: {model_class}")
+            return ModelType.UNKNOWN
+
+    def generate_model_script(self, target_column: str, feature_list: list[str], train_all_data: bool) -> str:
         """Fill in the model template with specific target and feature_list
         Args:
             target_column (str): Column name of the target variable
             feature_list (list[str]): A list of columns for the features
-            model_type (ModelType): ModelType.REGRESSOR or ModelType.CLASSIFIER
             train_all_data (bool): Train on ALL (100%) of the data
         Returns:
            str: The name of the generated model script
         """
 
         # FIXME: Revisit all of this since it's a bit wonky
-        script_name = "generated_xgb_model.py"
-        dir_path = Path(__file__).parent.absolute()
-        self.model_script_dir = os.path.join(dir_path, "light_model_harness")
-        template_path = os.path.join(self.model_script_dir, "xgb_model.template")
-        output_path = os.path.join(self.model_script_dir, script_name)
-        with open(template_path, "r") as fp:
-            xgb_template = fp.read()
+        # Did they specify a Scikit-Learn model class?
+        if self.model_class:
+            self.log.info(f"Using Scikit-Learn model class: {self.model_class}")
+            script_name = "generated_scikit_model.py"
+            dir_path = Path(__file__).parent.absolute()
+            self.model_script_dir = os.path.join(dir_path, "light_scikit_learn")
+            template_path = os.path.join(self.model_script_dir, "scikit_learn.template")
+            output_path = os.path.join(self.model_script_dir, script_name)
+            with open(template_path, "r") as fp:
+                scikit_template = fp.read()
 
-        # Template replacements
-        xgb_script = xgb_template.replace("{{target_column}}", target_column)
-        feature_list_str = json.dumps(feature_list)
-        xgb_script = xgb_script.replace("{{feature_list}}", feature_list_str)
-        xgb_script = xgb_script.replace("{{model_type}}", model_type)
-        metrics_s3_path = f"{self.model_training_root}/{self.output_uuid}"
-        xgb_script = xgb_script.replace("{{model_metrics_s3_path}}", metrics_s3_path)
-        xgb_script = xgb_script.replace("{{train_all_data}}", str(train_all_data))
+            # Template replacements
+            aws_script = scikit_template.replace("{{model_class}}", self.model_class)
+            aws_script = aws_script.replace("{{target_column}}", target_column)
+            feature_list_str = json.dumps(feature_list)
+            aws_script = aws_script.replace("{{feature_list}}", feature_list_str)
+            aws_script = aws_script.replace("{{model_type}}", self.model_type.value)
+            metrics_s3_path = f"{self.model_training_root}/{self.output_uuid}"
+            aws_script = aws_script.replace("{{model_metrics_s3_path}}", metrics_s3_path)
+            aws_script = aws_script.replace("{{train_all_data}}", str(train_all_data))
+
+        elif self.model_type == ModelType.REGRESSOR or self.model_type == ModelType.CLASSIFIER:
+            script_name = "generated_xgb_model.py"
+            dir_path = Path(__file__).parent.absolute()
+            self.model_script_dir = os.path.join(dir_path, "light_xgb_model")
+            template_path = os.path.join(self.model_script_dir, "xgb_model.template")
+            output_path = os.path.join(self.model_script_dir, script_name)
+            with open(template_path, "r") as fp:
+                xgb_template = fp.read()
+
+            # Template replacements
+            aws_script = xgb_template.replace("{{target_column}}", target_column)
+            feature_list_str = json.dumps(feature_list)
+            aws_script = aws_script.replace("{{feature_list}}", feature_list_str)
+            aws_script = aws_script.replace("{{model_type}}", self.model_type.value)
+            metrics_s3_path = f"{self.model_training_root}/{self.output_uuid}"
+            aws_script = aws_script.replace("{{model_metrics_s3_path}}", metrics_s3_path)
+            aws_script = aws_script.replace("{{train_all_data}}", str(train_all_data))
+
+        elif self.model_type == ModelType.QUANTILE_REGRESSOR:
+            script_name = "generated_quantile_model.py"
+            dir_path = Path(__file__).parent.absolute()
+            self.model_script_dir = os.path.join(dir_path, "light_quant_regression")
+            template_path = os.path.join(self.model_script_dir, "quant_regression.template")
+            output_path = os.path.join(self.model_script_dir, script_name)
+            with open(template_path, "r") as fp:
+                quant_template = fp.read()
+
+            # Template replacements
+            aws_script = quant_template.replace("{{target_column}}", target_column)
+            feature_list_str = json.dumps(feature_list)
+            aws_script = aws_script.replace("{{feature_list}}", feature_list_str)
+            metrics_s3_path = f"{self.model_training_root}/{self.output_uuid}"
+            aws_script = aws_script.replace("{{model_metrics_s3_path}}", metrics_s3_path)
 
         # Now write out the generated model script and return the name
         with open(output_path, "w") as fp:
-            fp.write(xgb_script)
+            fp.write(aws_script)
+
+        # Now we make sure the model script dir only has template, model script, and a requirements file
+        for file in os.listdir(self.model_script_dir):
+            if file not in [script_name, "requirements.txt"] and not file.endswith(".template"):
+                self.log.warning(f"Finding {file} in model_script_dir...")
         return script_name
 
     def transform_impl(
@@ -160,19 +255,19 @@ class FeaturesToModel(Transform):
         self.log.important(f"Feature List for Modeling: {self.model_feature_list}")
 
         # Generate our model script
-        script_path = self.generate_model_script(
-            self.target_column, self.model_feature_list, self.model_type.value, train_all_data
-        )
+        script_path = self.generate_model_script(self.target_column, self.model_feature_list, train_all_data)
 
-        # Metric Definitions for Regression and Classification
-        if self.model_type == ModelType.REGRESSOR:
+        # Metric Definitions for Regression
+        if self.model_type == ModelType.REGRESSOR or self.model_type == ModelType.QUANTILE_REGRESSOR:
             metric_definitions = [
                 {"Name": "RMSE", "Regex": "RMSE: ([0-9.]+)"},
                 {"Name": "MAE", "Regex": "MAE: ([0-9.]+)"},
                 {"Name": "R2", "Regex": "R2: ([0-9.]+)"},
                 {"Name": "NumRows", "Regex": "NumRows: ([0-9]+)"},
             ]
-        else:
+
+        # Metric Definitions for Classification
+        elif self.model_type == ModelType.CLASSIFIER:
             # We need to get creative with the Classification Metrics
 
             # Grab all the target column class values (class labels)
@@ -201,6 +296,11 @@ class FeaturesToModel(Transform):
                         {"Name": f"ConfusionMatrix:{row}:{col}", "Regex": f"ConfusionMatrix:{row}:{col} ([0-9.]+)"}
                     )
 
+        # If the model type is UNKNOWN, our metric_definitions will be empty
+        else:
+            self.log.warning(f"ModelType is {self.model_type}, skipping metric_definitions...")
+            metric_definitions = []
+
         # Create a Sagemaker Model with our script
         self.estimator = SKLearn(
             entry_point=script_path,
@@ -244,7 +344,7 @@ class FeaturesToModel(Transform):
             output_model.set_class_labels(self.class_labels)
 
         # Call the Model onboard method
-        output_model.onboard(interactive=False)
+        output_model.onboard_with_args(self.model_type, self.target_column, self.model_feature_list)
 
     def create_and_register_model(self):
         """Create and Register the Model"""
@@ -260,10 +360,13 @@ class FeaturesToModel(Transform):
         )
 
         # Register our model
+        image = InferenceImage.get_image_uri(self.sm_session.boto_region_name, "sklearn", "1.2.1")
+        self.log.important(f"Registering model {self.output_uuid} with image {image}...")
         model = self.estimator.create_model(role=self.sageworks_role_arn)
         model.register(
             model_package_group_name=self.output_uuid,
             framework_version="1.2.1",
+            image_uri=image,
             content_types=["text/csv"],
             response_types=["text/csv"],
             inference_instances=["ml.t2.medium"],
@@ -276,16 +379,47 @@ class FeaturesToModel(Transform):
 if __name__ == "__main__":
     """Exercise the FeaturesToModel Class"""
 
-    # Create the class with inputs and outputs and invoke the transform
+    # Regression Model
     input_uuid = "abalone_features"
     output_uuid = "abalone-regression"
     to_model = FeaturesToModel(input_uuid, output_uuid, ModelType.REGRESSOR)
     to_model.set_output_tags(["abalone", "public"])
-    to_model.transform(target_column="class_number_of_rings", description="Abalone Regression Model")
+    to_model.transform(target_column="class_number_of_rings", description="Abalone Regression")
 
-    # Now a classification model
+    """
+    # Classification Model
     input_uuid = "wine_features"
     output_uuid = "wine-classification"
     to_model = FeaturesToModel(input_uuid, output_uuid, ModelType.CLASSIFIER)
     to_model.set_output_tags(["wine", "public"])
-    to_model.transform(target_column="wine_class", description="Wine Classification Model")
+    to_model.transform(target_column="wine_class", description="Wine Classification")
+    """
+
+    # Quantile Regression Model (Abalone)
+    """
+    input_uuid = "abalone_features"
+    output_uuid = "abalone-quantile-reg"
+    to_model = FeaturesToModel(input_uuid, output_uuid, ModelType.QUANTILE_REGRESSOR)
+    to_model.set_output_tags(["abalone", "quantiles"])
+    to_model.transform(target_column="class_number_of_rings", description="Abalone Quantile Regression")
+    """
+
+    # Scikit-Learn KNN Regression Model (Abalone)
+    """
+    input_uuid = "abalone_features"
+    output_uuid = "abalone-knn-reg"
+    to_model = FeaturesToModel(input_uuid, output_uuid, model_class="KNeighborsRegressor")
+    to_model.set_output_tags(["abalone", "knn"])
+    new_model = to_model.transform(
+        target_column="class_number_of_rings", description="Abalone KNN Regression", train_all_data=True
+    )
+
+    # Scikit-Learn Random Forest Classification Model (Wine)
+    input_uuid = "wine_features"
+    output_uuid = "wine-rfc-class"
+    to_model = FeaturesToModel(input_uuid, output_uuid, model_class="RandomForestClassifier")
+    to_model.set_output_tags(["wine", "rfc"])
+    new_model = to_model.transform(
+        target_column="wine_class", description="Wine RF Classification", train_all_data=True
+    )
+    """
