@@ -13,12 +13,40 @@ from awswrangler.exceptions import NoFilesFound
 from pathlib import Path
 import posixpath
 from sagemaker.session import Session as SageSession
+from sagemaker import image_uris
 from collections.abc import Mapping, Iterable
 
 # SageWorks Logger
 log = logging.getLogger("sageworks")
 
 
+def get_image_uri_with_digest(framework, region, version, sm_session: SageSession):
+    # Retrieve the base image URI using sagemaker SDK
+    base_image_uri = image_uris.retrieve(
+        framework=framework, region=region, version=version, sagemaker_session=sm_session
+    )
+    print(f"Base Image URI: {base_image_uri}")
+
+    # Extract repository name and image tag from the base image URI
+    repo_uri, image_tag = base_image_uri.split(":")
+    repository_name = repo_uri.split("/")[-1]
+
+    # Use AWS CLI to get image details and find the digest
+    ecr_client = sm_session.boto_session.client("ecr", region_name=region)
+    response = ecr_client.describe_images(
+        repositoryName=repository_name,
+        imageIds=[
+            {"imageTag": image_tag},
+        ],
+    )
+    if "imageDetails" in response and len(response["imageDetails"]) > 0:
+        image_digest = response["imageDetails"][0]["imageDigest"]
+        full_image_uri = f"{repo_uri}@{image_digest}"
+        return full_image_uri
+    else:
+        raise ValueError("Image details not found for the specified tag.")
+
+
 def client_error_info(err: botocore.exceptions.ClientError):
     """Helper method to get information about a botocore.exceptions.ClientError"""
     error_code = err.response["Error"]["Code"]
@@ -35,7 +63,7 @@ def client_error_info(err: botocore.exceptions.ClientError):
     log.error(f"Request ID: {request_id}")
 
 
-def list_tags_with_throttle(arn: str, sm_session: SageSession) -> dict:
+def list_tags_with_throttle(arn: str, sm_session) -> dict:
     """A Wrapper around SageMaker's list_tags method that handles throttling
     Args:
         arn (str): The ARN of the SageMaker resource
@@ -48,10 +76,11 @@ def list_tags_with_throttle(arn: str, sm_session: SageSession) -> dict:
 
     # Log the call
     log.debug(f"Calling list_tags for {arn}...")
-    sleep_time = 0.25
+    sleep_times = [0.25, 0.5, 1, 2, 4, 8]
+    max_attempts = len(sleep_times)
 
-    # Loop 4 times with exponential backoff
-    for i in range(4):
+    # Loop with exponential backoff
+    for attempt in range(max_attempts):
         try:
             # Call the AWS List Tags method
             aws_tags = sm_session.list_tags(arn)
@@ -60,13 +89,23 @@ def list_tags_with_throttle(arn: str, sm_session: SageSession) -> dict:
 
         except botocore.exceptions.ClientError as e:
             error_code = e.response["Error"]["Code"]
+            error_message = e.response["Error"]["Message"]
+
             if error_code == "ThrottlingException":
-                log.warning(f"ThrottlingException: list_tags on {arn}")
-                time.sleep(sleep_time)
-                sleep_time *= 2
+                log.info(f"ThrottlingException: list_tags on {arn}")
+                time.sleep(sleep_times[attempt])
+
+            elif error_code == "ValidationException":
+                if "does not exist" in error_message:
+                    log.warning(f"ValidationException: {arn} does not exist")
+                    return {}
+                else:
+                    log.error(f"ValidationException: {error_message}")
+                    raise e
+
             else:
                 # Handle other ClientErrors that may occur
-                log.error(f"Unexpected ClientError: {error_code}")
+                log.error(f"Unexpected ClientError: {error_code} - {error_message}")
                 raise e
 
     # If we get here, we've failed to retrieve the tags
@@ -332,6 +371,10 @@ def pull_s3_data(s3_path: str, embedded_index=False) -> Union[pd.DataFrame, None
     except NoFilesFound:
         log.info(f"Could not find S3 data at {s3_path}...")
         return None
+    except Exception as e:
+        log.error(f"Failed to pull data from {s3_path}!")
+        log.error(e)
+        return None
 
 
 def compute_size(obj: object) -> int:
@@ -396,14 +439,29 @@ if __name__ == "__main__":
     """Exercise the AWS Utils"""
     from pprint import pprint
     from sageworks.core.artifacts.feature_set_core import FeatureSetCore
+    from sageworks.aws_service_broker.aws_account_clamp import AWSAccountClamp
+
+    # Grab out SageMaker Session from the AWS Account Clamp
+    sm_session = AWSAccountClamp().sagemaker_session()
+    boto3_session = AWSAccountClamp().boto_session()
 
     my_features = FeatureSetCore("test_features")
     my_meta = my_features.sageworks_meta()
     pprint(my_meta)
 
+    # Get the image URI with digest
+    framework = "sklearn"
+    region = "us-west-2"
+    version = "1.2-1"
+
+    try:
+        full_image_uri = get_image_uri_with_digest(framework, region, version, sm_session)
+        print(f"Full Image URI with Digest: {full_image_uri}")
+    except Exception as e:
+        print(f"Error: {e}")
+
     # Test the newest files in an S3 folder method
     s3_path = "s3://sandbox-sageworks-artifacts/endpoints/inference/abalone-regression-end"
-    sm_session = SageSession()
     most_recent = newest_files([s3_path], sm_session)
 
     # Add a health tag
