@@ -5,13 +5,6 @@ from collections import defaultdict
 import time
 
 
-# Check if we're running on Docker
-# AWS Cloud Watch doesn't like ANSI escape codes or custom log levels
-from sageworks.utils.docker_utils import running_on_ecs
-
-on_ecs = running_on_ecs()
-
-
 class ThrottlingFilter(logging.Filter):
     def __init__(self, rate_seconds=60):
         super().__init__()
@@ -69,7 +62,7 @@ class ColoredFormatter(logging.Formatter):
         "IMPORTANT": "\x1b[38;5;113m",  # LightGreen
         "WARNING": "\x1b[38;5;220m",  # DarkYellow
         "ERROR": "\x1b[38;5;208m",  # Orange
-        "CRITICAL": "\x1b[38;5;198m",  # Red
+        "CRITICAL": "\x1b[38;5;198m",  # Hot Pink
     }
     COLORS_LIGHT_THEME = {
         "DEBUG": "\x1b[38;5;21m",  # Blue
@@ -90,15 +83,8 @@ class ColoredFormatter(logging.Formatter):
 
 
 def logging_setup(color_logs=True):
-    """Setup the logging for the application.""
+    """Setup the logging for the application."""
 
-    Args:
-        color_logs (bool, optional): Whether to colorize the logs. Defaults to True.
-    """
-
-    # Cloud Watch doesn't like colors
-    if on_ecs:
-        color_logs = False
     log = logging.getLogger("sageworks")
 
     # Turn off propagation to root logger
@@ -106,7 +92,6 @@ def logging_setup(color_logs=True):
 
     # Remove any existing handlers
     while log.handlers:
-        log.info("Removing log handler...")
         log.removeHandler(log.handlers[0])
 
     # Setup new handler
@@ -132,8 +117,9 @@ def logging_setup(color_logs=True):
         log.debug("Debugging enabled via SAGEWORKS_DEBUG environment variable.")
     else:
         log.setLevel(logging.INFO)
-        throttle_filter = ThrottlingFilter(rate_seconds=5)
-        handler.addFilter(throttle_filter)
+        # Note: Not using the ThrottlingFilter for now
+        # throttle_filter = ThrottlingFilter(rate_seconds=5)
+        # handler.addFilter(throttle_filter)
 
     # Suppress specific logger
     logging.getLogger("sagemaker.config").setLevel(logging.WARNING)
