@@ -20,10 +20,6 @@ def test():
     check_boto_session = aws_account_clamp.boto_session()
     print("Assume Role Success...")
 
-    print("*** AWS App Config Check ***")
-    aws_account_clamp.check_app_config(check_boto_session)
-    print("App Config Check Success...")
-
     print("*** AWS S3 Access Check ***")
     aws_account_clamp.check_s3_access(check_boto_session)
     print("S3 Access Check Success...")
